from .common import * 
from bbclient import *

params = [
    ([],),
    (["doc"],),
    (["doc", "dummy"],)
]

@pytest.mark.parametrize("flags", params)
def test_generate_targets_tree_main(main_client: BBClient, flags: List[str]) -> None:
    __test_impl(main_client, flags)

@pytest.mark.parametrize("flags", params)
def test_generate_targets_tree_kirkstone(kirkstone_client: BBClient, flags: List[str]) -> None:
    __test_impl(kirkstone_client, flags)

@pytest.mark.parametrize("flags", params)
def test_generate_targets_tree_dunfell(dunfell_client: BBClient, flags: List[str]) -> None:
    __test_impl(dunfell_client, flags)

def __test_impl(client: BBClient, flags: List[str]) -> None:
    ret: List[getAllKeysWithFlagsResult] = client.get_all_keys_with_flags(flags)
    assert ret is not None


from .common import * 
from bbclient import *

params = [
    ("getVar", ["MACHINE"]),
]

@pytest.mark.parametrize("command, args", params)
def test_data_store_connector_cmd_main(main_client: BBClient, command: str, args: List[Any]) -> None:
    __test_imple(main_client, command, args)

@pytest.mark.parametrize("command, args", params)
def test_data_store_connector_cmd_kirkstone(kirkstone_client: BBClient, command: str, args: List[Any]) -> None:
    __test_imple(kirkstone_client, command, args)

@pytest.mark.parametrize("command, args", params)
def test_data_store_connector_cmd_dunfell(dunfell_client: BBClient, command: str, args: List[Any]) -> None:
    __test_imple(dunfell_client, command, args)

def __test_imple(client: BBClient, command: str, args: List[Any]) -> None:
    ret: Any = client.data_store_connector_cmd(0, command, *args)
    print(ret)
    assert isinstance(ret, str)

# tests for getVarFlags and getVarFlag
def test_get_var_flags_main(main_client: BBClient) -> None:
    __test_imple_for_get_var_flags(main_client)

def test_get_var_flags_kirkstone(kirkstone_client: BBClient) -> None:
    __test_imple_for_get_var_flags(kirkstone_client)

def test_get_var_flags_dunfell(dunfell_client: BBClient) -> None:
    __test_imple_for_get_var_flags(dunfell_client)

def __test_imple_for_get_var_flags(client: BBClient) -> None:
    ret: Mapping[str, str] = client.data_store_connector_cmd(0, "getVarFlags", "MACHINE")
    for key, value in ret.items():
        expect: Mapping[str, str] = client.data_store_connector_cmd(0, "getVarFlag", "MACHINE", key)
        assert value == expect


from .common import * 
from bbclient import *

def test_compare_revisions(main_client: BBClient) -> None:
    __test_imple(main_client)

def test_build_file_kirkstone(kirkstone_client: BBClient) -> None:
    __test_imple(kirkstone_client)

def test_build_file_dunfell(dunfell_client: BBClient) -> None:
    __test_imple(dunfell_client)

def __test_imple(client: BBClient) -> None:
    client.compare_revisions()
    result: Optional[BBEventBase] = client.wait_done_async()
    assert isinstance(result, CommandCompletedEvent)


from .common import *
from bbclient import *

def test_set_event_mask(main_client: BBClient) -> None:
    ui_handler: int = main_client.get_uihandler_num()
    ret: bool = main_client.set_event_mask(ui_handler, logging.DEBUG, {}, ["*"])
    assert ret == True


from .common import *
from bbclient import *

params = [
    ("DUMMY", r"${MACHINE}"),
]

@pytest.mark.parametrize("variable_name, variable_value", params)
def test_disable_data_tracking_main(main_client: BBClient, variable_name, variable_value) -> None:
    __test_impl(main_client, variable_name, variable_value)

@pytest.mark.parametrize("variable_name, variable_value", params)
def test_disable_data_tracking_kirkstone(kirkstone_client: BBClient, variable_name, variable_value) -> None:
    __test_impl(kirkstone_client, variable_name, variable_value)

@pytest.mark.parametrize("variable_name, variable_value", params)
def test_disable_data_tracking_dunfell(dunfell_client: BBClient, variable_name, variable_value) -> None:
    __test_impl(dunfell_client, variable_name, variable_value)

def __test_impl(client: BBClient, variable_name, variable_value) -> None:
    client.enable_data_tracking()
    client.set_variable(variable_name, variable_value)
    ret = client.data_store_connector_varhist_cmd(0, "variable", variable_name)
    assert len(ret) == 1
    client.disable_data_tracking()
    client.set_variable(variable_name, variable_value)
    ret = client.data_store_connector_varhist_cmd(0, "variable", variable_name)
    assert len(ret) == 1
    ret = client.data_store_connector_varhist_cmd(0, "del_var_history", variable_name)


from .common import *
from bbclient import *

def test_get_runtime_recommends_main(main_client: BBClient) -> None:
    __test_impl(main_client)

def test_get_runtime_recommends_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_get_runtime_recommends_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    variable_name: str = "DUMMY"
    variable_value: str = r"${MACHINE}"
    client.set_variable(variable_name, variable_value)

    ret: str = client.get_set_variable(variable_name)
    machine_variable_name: str = "MACHINE"
    machine_variable_value: str = client.get_variable(machine_variable_name)
    assert ret == machine_variable_value

    ret: str = client.get_variable(variable_name, False)
    assert ret == machine_variable_value


from .common import * 
from bbclient import *

params = [
    (["busybox"], "fetch", CommandCompletedEvent),
    (["busybox", "python3:patch"], "fetch", CommandFailedEvent),
    (["busybox", "python3:do_patch"], "fetch", CommandCompletedEvent),
    (["busybox", "dummy_package"], "fetch", CommandFailedEvent),
]

@pytest.mark.parametrize("targets, task, expect", params)
def test_build_targets_main(main_client: BBClient, targets: List[str], task: str, expect: Type[BBEventBase]) -> None:
    __test_impl(main_client, targets, task, expect)

@pytest.mark.parametrize("targets, task, expect", params)
def test_build_targets_kirkstone(kirkstone_client: BBClient, targets: List[str], task: str, expect: Type[BBEventBase]) -> None:
    __test_impl(kirkstone_client, targets, task, expect)

@pytest.mark.parametrize("targets, task, expect", params)
def test_build_targets_dunfell(dunfell_client: BBClient, targets: List[str], task: str, expect: Type[BBEventBase]) -> None:
    __test_impl(dunfell_client, targets, task, expect)

def __test_impl(client: BBClient, targets: List[str], task: str, expect: Type[BBEventBase]) -> None:
    client.build_targets(targets, task)
    result: Optional[BBEventBase] = client.wait_done_async()
    assert isinstance(result, expect)


from .common import * 
from bbclient import *

params = [
    ("busybox"),
]

@pytest.mark.parametrize("target", params)
def test_get_file_appends_main(main_client: BBClient, target: str) -> None:
    __test_impl(main_client, target)

@pytest.mark.parametrize("target", params)
def test_get_file_appends_kirkstone(kirkstone_client: BBClient, target: str) -> None:
    __test_impl(kirkstone_client, target)

@pytest.mark.parametrize("target", params)
def test_get_file_appends_dunfell(dunfell_client: BBClient, target: str) -> None:
    __test_impl(dunfell_client, target)

def __test_impl(client: BBClient, target: str) -> None:
    ret: List[str] = client.find_best_provider(target)
    target_recipe_file_path: str = ret[3]
    ret: List[getAllKeysWithFlagsResult] = client.get_file_appends(target_recipe_file_path)
    assert ret is not None
    assert len(ret) != 0


from .common import *
from bbclient import *

params = [
    ("gcc"),
]

@pytest.mark.parametrize("package", params)
def test_generate_targets_tree_main(main_client: BBClient, package: str) -> None:
    __test_impl(main_client, package)

@pytest.mark.parametrize("package", params)
def test_generate_targets_tree_kirkstone(kirkstone_client: BBClient, package: str) -> None:
    __test_impl(kirkstone_client, package)

@pytest.mark.parametrize("package", params)
def test_generate_targets_tree_dunfell(dunfell_client: BBClient, package: str) -> None:
    __test_impl(dunfell_client, package)

def __test_impl(client: BBClient, package: str) -> None:
    ret: List[str] = client.find_best_provider(package)
    assert len(ret) == 4
    assert os.path.isfile(ret[3])
    datastore_index: int = client.parse_recipe_file(ret[3])
    result: str = client.data_store_connector_cmd(datastore_index, "getVar", "PN")
    assert result == package


from .common import * 
from bbclient import *

def test_get_overlayed_recipes_main(main_client: BBClient) -> None:
    __test_impl(main_client)

def test_get_overlayed_recipes_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_get_overlayed_recipes_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    ret: Any = client.get_overlayed_recipes()
    # TODO: assert


from .common import * 
from bbclient import *

def test_get_recipe_inherits_main(main_client: BBClient) -> None:
    __test_impl(main_client)

def test_get_recipe_inherits_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_get_recipe_inherits_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    ret: List[GetRecipeInheritsResult] = client.get_recipe_inherits()
    assert ret is not None
    assert len(ret) != 0


#!/usr/bin/env python3
""" 
This file provides client class for bitbake server.
"""

__author__ = "AngryMane"
__authors__ = ["AngryMane"]
__contact__ = "regulationdango@gmail.com"
__copyright__ = "Copyright 2022"
__credits__ = ["AngryMane"]
__date__ = "2022/07/31"
__deprecated__ = False
__email__ = "regulationdango@gmail.com"
__license__ = "MIT"
__maintainer__ = "AngryMane"
__status__ = "in progress"
__version__ = "0.0.1"

import os
import sys
import time
import subprocess
from logging import Logger
from functools import wraps
from typing import Any, List, Optional, Tuple, Mapping, Callable, Iterable, Type

from .bbcommon import *
from .bbevent import *


class BBClient:
    """Client for bitbake RPC Server

    Attributes:
        project_path (str): poky directory path
        __is_server_running (bool): server is running or not
        __server_connection (bb.server.xmlrpcclient.BitBakeXMLRPCServerConnection): touch point to server
        __logger (Optional[Logger]): Logger instance for debugging. Default is None.
    """

    def logger_decorator(func: Callable): # type: ignore
        """Decorator for logging

        Args:
            func (Callable): target function

        Note:
            This is just for debuggging.
        """
        @wraps(func)
        def inner_function(self: "BBClient", *args, **kwargs):
            if self.__logger:
                self.__logger.debug(f"{func.__name__} start.")
            ret = func(self, *args, **kwargs)
            if self.__logger:
                self.__logger.debug(f"{func.__name__} end.")
            return ret
        return inner_function

    # --- setup functions ---
    def __init__(
        self: "BBClient", project_abs_path: str, init_script_path: str = ":", logger: Optional[Logger] = None
    ) -> None:
        """Initialize BBClient instance

        Args:
            self (BBClient): none
            project_abs_path (str): abslute path to bitbake project, basically poky dir.
            init_script_path (str): initialize bitbake proejct command running at project_abs_path. This is maybe ". oe-init-build-env". if you already executed initialize command, you don't need to input this.
            logger (Logger): logger instance for debuggind. Default is None.
        """
        self.project_path: str = project_abs_path
        self.__is_server_running: bool = False
        self.__logger: Optional[Logger] = logger
        pipe: subprocess.Popen = subprocess.Popen(
            f"{init_script_path} > /dev/null; env",
            stdout=subprocess.PIPE,
            shell=True,
            cwd=self.project_path,
            executable="/bin/bash",
            text=True,
        )
        output, _ = pipe.communicate()
        env = dict((line.split("=", 1) for line in output.splitlines()))
        os.environ.update(env)

    def __del__(self: "BBClient") -> None:
        """Finalize BBClient instance

        Args:
            self (BBClient): none
        """
        self.stop_server()

    @logger_decorator
    def start_server(self: "BBClient") -> None:
        """Start bitbake XML RPC server

        Args:
            self (BBClient): none

        Note:
            Remote server support deprecated becuase bitbake has some minor software bug when using remote server.
        """
        connection, _ = self.__connect_server(
            self.project_path
        )
        self.__server_connection = connection
        self.__is_server_running = True

    @logger_decorator
    def stop_server(self: "BBClient") -> None:
        """Stop bitbake XML RPC server

        Args:
            self (BBClient): none
        """
        if not self.__is_server_running:
            return
        self.state_shutdown()
        time.sleep(1)
        self.state_force_shutdown()
        time.sleep(2)
        self.__server_connection.connection.terminateServer()
        self.__server_connection.terminate()
        self.__is_server_running = False

    # --- utility functions ---
    @logger_decorator
    def wait_done_async(self: "BBClient", timeout: Optional[float] = None) -> Optional[BBEventBase]:
        """Wait CommandCompletedEvent, CommandExitEvent, CommandFailedEvent event

        Args:
            self (BBClient): none
            timeout (Optional[float], optional): timeout. (seconds)

        Returns:
            Optional[BBEventBase]: target event or None
        """
        return self.wait_event([CommandCompletedEvent, CommandExitEvent, CommandFailedEvent], timeout) # type: ignore

    @logger_decorator
    def wait_event(self: "BBClient", event_types: List[Type[BBEventBase]], timeout: Optional[float] = None) -> Optional[BBEventBase]:
        """Wait specific event

        Args:
            self (BBClient): none
            event_types (List[Type[BBEventBase]]): event types you wait for. BBEventBase and its inherits types is defined in bbcommon.py.
            timeout (float): timeout. (seconds)

        Returns:
            Optional[BBEventBase]: The event you wait for or None
        
        Note:
            | This function will pop events from event queue. This event queue is reused between many commands, so this queue may have events from previous commands.
            | When you wait bb.command.CommandCompleted event, please confirm there is no left bb.command.CommandCompleted event from previous command.
        """
        # f"<class '{event_name}'>" -> event_name
        start_time: float = time.perf_counter()
        ret: Optional[BBEventBase] = None
        while True:
            cur_event: Optional[BBEventBase] = self.get_event(0.01)
            find_matched_type: Optional[Type[BBEventBase]] = next(filter(lambda x: isinstance(cur_event, x), event_types), None) # type: ignore
            is_instance_of_target: bool = True if find_matched_type else False
            if is_instance_of_target:
                ret = cur_event
                break
            execution_time: float = time.perf_counter() - start_time
            if timeout and timeout < execution_time:
                if self.__logger:
                    self.__logger.warning(f"Timeout occurred because {execution_time} second has elapsed")
                break
        return ret

    def get_event(self: "BBClient", timeout: Optional[float] = None) -> Optional[BBEventBase]:
        """Get oldest event

        Args:
            self (BBClient): none
            timeout (Optional[float]): timeout. if timeout, return None

        Returns:
            Optional[BBEventBase]: event notification objects. See bbcommon.py
        """
        cur_event: Any = self.__server_connection.events.waitEvent(timeout)
        if not cur_event:
            return None
        cur_event_name: str = str(type(cur_event))[8:-2]
        itr: Iterable = filter(lambda x: x.is_target(cur_event_name), ALL_BB_EVENTS)
        event_class: Optional[Type[BBEventBase]] = next(itr, None) # type: ignore
        ret: BBEventBase = event_class(cur_event.__dict__) if event_class else UnknownEvent(cur_event_name, cur_event.__dict__)
        if not isinstance(ret, UnknownEvent) and self.__logger:
            self.__logger.debug(f"get {cur_event_name}: {ret.__dict__}")
        if isinstance(ret, UnknownEvent) and self.__logger:
            self.__logger.debug(f"get Unknow event {cur_event_name}: {ret.__dict__}")
        return ret
        

    # --- bitbake server sync functions  ---
    @logger_decorator
    def state_shutdown(self: "BBClient") -> None:
        """Terminate tasks

        Args:
            self (BBClient): none

        Note:
            Terminate tasks defined in recipes. If there are running tasks, wait for them to exit.
        """
        self.__run_command(self.__server_connection, "stateShutdown", logger=self.__logger)

    @logger_decorator
    def state_force_shutdown(self: "BBClient") -> None:
        """Terminate tasks

        Args:
            self (BBClient): none

        Note:
            Terminate tasks defined in recipes. If there are running tasks, terminate them.
        """
        self.__run_command(self.__server_connection, "stateForceShutdown", logger=self.__logger)

    @logger_decorator
    def get_all_keys_with_flags(
        self: "BBClient", flag_list: List[str]
    ) -> List[getAllKeysWithFlagsResult]:
        """Get value, history and specified flags of all global variables.

        Args:
            self (BBClient): none
            flag_list(List[str]): Target flags. If flags are unnecessary, please set [].

        Returns:
            List[getAllKeysWithFlagsResult]: See getAllKeysWithFlagsResult.

        Note:
            If you want to get variables defined at any recipes, please use `parse_recipe_file` and `data_store_connector_cmd`.
        """
        ret: Mapping = self.__run_command(  # type: ignore
            self.__server_connection, "getAllKeysWithFlags", flag_list, logger=self.__logger
        )
        return [getAllKeysWithFlagsResult(key, value) for key, value in ret.items()]

    @logger_decorator
    def get_variable(self: "BBClient", name: str, expand: bool = True) -> str:
        """Get variable value

        Args:
            self (BBClient): none
            name (str): variable name
            expand (bool, optional): Whether to expand references to other variables. Defaults to True.

        Returns:
            str: variable value
        """
        expand_str: str = "True" if expand else "False" # bitbake decide whether or not to expand variable by expand == "True" 
        return self.__run_command(self.__server_connection, "getVariable", name, expand_str, logger=self.__logger)  # type: ignore

    @logger_decorator
    def set_variable(self: "BBClient", name: str, value: str) -> None:
        """Set vaiable value

        Args:
            self (BBClient): none
            name (str): variable name
            value (str): variable value you want to set

        """
        self.__run_command(self.__server_connection, "setVariable", name, value, logger=self.__logger)

    @logger_decorator
    def get_set_variable(self: "BBClient", name: str, expand: bool = True) -> str:
        """Get variable from cache and set it into cache

        Args:
            self (BBClient): none
            name (str): variable name
            expand (bool, optional): Whether to expand references to other variables. Defaults to True.

        Returns:
            str: variable value

        Note:
            This is maybe for expand variable value in cache.
        """
        expand_str: str = "True" if expand else "False" # bitbake decide whether or not to expand variable by expand == "True" 
        return self.__run_command(  # type: ignore
            self.__server_connection, "getSetVariable", name, expand_str, logger=self.__logger
        )

    @logger_decorator
    def set_config(self: "BBClient", name: str, value: str) -> None:
        """Set CookerConfigcation properties

        Args:
            self (BBClient): none
            name (str): property name
            value (str): property value

        Note:
            | If you want to know all CookerConfigcation properties, see poky/bitbake/lib/bb/cookerdata.py. But I don't know the detail and how to use it...
        """
        self.__run_command(self.__server_connection, "setConfig", name, value, logger=self.__logger)

    @logger_decorator
    def enable_data_tracking(self: "BBClient") -> None:
        """Enable data tracking

        Args:
            self (BBClient): none

        Note:
            | If enable, cooker cacheata(VariableHistory class) logs the history of changin value. You can see the log by dataStoreConnectorVarHistCmdEmit command.
        """
        self.__run_command(self.__server_connection, "enableDataTracking", logger=self.__logger)

    @logger_decorator
    def disable_data_tracking(self: "BBClient") -> None:
        """Disable data tracking

        Args:
            self (BBClient): none

        Note:
            Please see enable_data_tracking command
        """
        self.__run_command(self.__server_connection, "disableDataTracking", logger=self.__logger)

    @logger_decorator
    def set_pre_post_conf_files(
        self: "BBClient", pre_files: str, post_files: str
    ) -> None:
        """Set pre-load files and post-load files of bitbake.conf at parse_configuration_files command

        Args:
            self (BBClient): none
            pre_files (str): files loaded before bitbake.conf
            post_files (str): files loaded after bitbake.conf

        Note:
            When parse_configuration_files command, pre_files will load before bitbake.conf and post_files will load after bitbake.conf.
        """
        self.__run_command(
            self.__server_connection, "setPrePostConfFiles", pre_files, post_files, logger=self.__logger
        )

    @logger_decorator
    def match_file(
        self: "BBClient", file_path_regex: str, mutli_conf_name: str = ""
    ) -> str:
        """Search file by regex

        Args:
            self (BBClient): none
            file_path_regex (str): search regex pattern
            mutli_conf_name (str): target multi-config. Defaults to ''.

        Returns:
            str: matched file path

        WARNING:
            This command will fail because the second parameter and the first one mixed up in the command.

        Note:
            | This command can extract only one file. If you input the regex matching to many file, this command will fail. 
        """

        return self.__run_command(  # type: ignore
            self.__server_connection, "matchFile", file_path_regex, mutli_conf_name, logger=self.__logger
        )

    @logger_decorator
    def get_uihandler_num(self: "BBClient") -> int:
        """Get ui handler num.

        Args:
            self (BBClient): none

        Returns:
            int: ui hanlder num
        """
        return self.__run_command(self.__server_connection, "getUIHandlerNum", logger=self.__logger)  # type: ignore

    @logger_decorator
    def set_event_mask(
        self: "BBClient",
        handler_num: int,
        log_level: int,
        debug_domains: Mapping[str, int],
        mask: List[str],
    ) -> bool:
        """Set log filter for specified ui handler

        Args:
            self (BBClient): none
            handler_num (int): target ui handler
            log_level (int): logging.DEBUG, logging.INFO, etc...
            debug_domains (Mapping[str, int]): logger name.
            mask (List[str]): target event name. If you don't want to filter by event name, put [ "*" ].

        Returns:
            bool: if handler_num is invalid, return False, otherwise True.

        Note:
            | debug_domains is a little bit complex. If you want to extract logging.getLogger("NAME"), put { "NAME": logging.INFO }. 
            | debug_domains value filters log level like log_level arg. log_level arg and debug_domains value are `or conditoin`.
            |
            | mask is also a little bit difficult. This args filters log by event class name like bb.event.BuildStarted, bb.command.CommandCompleted, etc..
            | It is unclear what type of logs are available.
        """
        # TODO: investigate how to use
        # log level : logging.DEBUG, logging.INFO, etc...
        return self.__run_command(  # type: ignore
            self.__server_connection,
            "setEventMask",
            handler_num,
            log_level,
            debug_domains,
            mask, 
            logger=self.__logger
        )

    @logger_decorator
    def set_features(self: "BBClient", features: List[BBFeature]) -> None:
        """Set feature(Enable feature)

        Args:
            self (BBClient): none
            features (List[BBFeature]): list of HOB_EXTRA_CACHES, BASEDATASTORE_TRACKING and SEND_SANITYEVENTS.

        Note:
            | if enable HOB_EXTRA_CACHES, recipecheces has extra-info like SUMMARY, LICENSE, DESCRIPTION, etc...
            | if enable BASEDATASTORE_TRACKING, enable_data_tracking. See enable_data_tracking command.
            | if enable SEND_SANITYEVENTS, this feature has not been implemented and is currently meaningless.
        """
        self.__run_command(
            self.__server_connection,
            "setFeatures",
            [feature.value for feature in features],
            logger=self.__logger
        )

    @logger_decorator
    def update_config(
        self: "BBClient",
        options: Mapping[str, Any],
        environment: Mapping[str, str],
        command_line: str,
    ) -> None:
        """Update config

        Args:
            self (BBClient): none
            options (Mapping[str, Any]): dict of prefile and postfile
            environment (Mapping[str, str]): environment variables
            command_line (str): will set to BB_CMDLINE

        Note:
            | options will set like {"prefile": ["FILE_NAME", "FILE_NAME"], "postfile": ["FILE_NAME", "FILE_NAME"]}
            | They are prefiles and postfiles settings. See set_pre_post_conf_files command. if you change prefiles or postfiles, bitbake will reset their cache.
            | environment is environment variables, like {"KEY": "VALUE", "KEY": "VALUE"}. if you add/change/delete any environemt variable, bitbake will reset their cache.
            | command_line will set to BB_CMDLINE variable. BB_CMDLINE seems not to be used at all.
        """
        self.__run_command(
            self.__server_connection, "updateConfig", options, environment, command_line, logger=self.__logger
        )

    @logger_decorator
    def parse_configuration(self: "BBClient") -> None:
        """Parse configuration

        Note:
            This command clears caches and re-builds them.
        """
        self.__run_command(self.__server_connection, "parseConfiguration", logger=self.__logger)

    @logger_decorator
    def get_layer_priorities(self: "BBClient") -> List[GetLayerPrioritiesResult]:
        """Get name, path, priority of all layers

        Args:
            self (BBClient): none

        Returns:
            List[GetLayerPrioritiesResult]: See GetLayerPrioritiesResult.

        WARNING:
            This command deletes caches(bug?).
        """
        ret: List[List[Any]] = self.__run_command(self.__server_connection, "getLayerPriorities", logger=self.__logger)  # type: ignore
        return [GetLayerPrioritiesResult(layer) for layer in ret]

    @logger_decorator
    def get_recipes(self: "BBClient", multi_config: str = "") -> List[GetRecipesResult]:
        """Get all package name from cache

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRecipesResult]: See GetRecipesResult.
        """
        ret: List[List[Any]] = self.__run_command(self.__server_connection, "getRecipes", multi_config, logger=self.__logger)  # type: ignore
        return [GetRecipesResult(recipe) for recipe in ret]

    @logger_decorator
    def get_recipe_depends(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRecipeDependsResult]:
        """Get recipe depends

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRecipeDependsResult]: See GetRecipeDependsResult.
        """
        ret: List[List[Any]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRecipeDepends", multi_config, logger=self.__logger
        )
        return [GetRecipeDependsResult(recipe_file) for recipe_file in ret]

    @logger_decorator
    def get_recipe_versions(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRecipeVersionsResult]:
        """Get all recipe versions

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRecipeVersions]: See GetRecipeVersions.
        """
        ret: Mapping[str, List[str]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRecipeVersions", multi_config, logger=self.__logger
        )
        return [GetRecipeVersionsResult(value, key) for key, value in ret.items()]

    @logger_decorator
    def get_recipe_provides(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRecipeProvidesResult]:
        """Get all recipe files and its packages

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRecipeProvidesResult]: See GetRecipeProvidesResult
        """
        ret: Mapping[str, List[str]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRecipeProvides", multi_config, logger=self.__logger
        )
        return [GetRecipeProvidesResult(key, value) for key, value in ret.items()]

    @logger_decorator
    def get_recipe_packages(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRecipePackagesResult]:
        """Get all recipe files and its recipe files

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRecipePackagesResult]: See GetRecipePackagesResult.
        """
        ret: Mapping[str, List[str]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRecipePackages", multi_config, logger=self.__logger
        )
        return [GetRecipePackagesResult(key, value) for key, value in ret.items()]

    @logger_decorator
    def get_recipe_packages_dynamic(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRecipePackagesDynamicResult]:
        """Get all recipe files that provides PACKAGE_DYNAMIC and its PACKAGE_DYNAMIC

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRecipePackagesDynamicResult]: See GetRecipePackagesDynamicResult
        """
        ret: Mapping[str, List[str]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRecipePackagesDynamic", multi_config, logger=self.__logger
        )
        return [
            GetRecipePackagesDynamicResult(key, value) for key, value in ret.items()
        ]

    @logger_decorator
    def get_r_providers(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRProvidersResult]:
        """Get alias of PN and its recipe files

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRProvidersResult]: See GetRProvidersResult

        Note:
            | If you want to know the detail of alias of PN, See `here <https://docs.yoctoproject.org/ref-manual/variables.html?highlight=rprovide#term-RPROVIDES>`_
        """
        ret: Mapping[str, List[str]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRProviders", multi_config, logger=self.__logger
        )
        return [GetRProvidersResult(key, value) for key, value in ret.items()]

    @logger_decorator
    def get_runtime_depends(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRuntimeDependsResult]:
        """Get all runtime dependency by all recipe files

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRuntimeDependsResult]: See GetRuntimeDependsResult
        """
        ret: List[str, Mapping[str, List[str]]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRuntimeDepends", multi_config, logger=self.__logger
        )
        return [GetRuntimeDependsResult(*data) for data in ret]

    @logger_decorator
    def get_runtime_recommends(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRuntimeRecommendsResult]:
        """Get all runtime recoomends(=weak depends) by all recipe files

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRuntimeRecommendsResult]: See GetRuntimeRecommendsResult
        """
        ret: List[str, Mapping[str, List[str]]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRuntimeRecommends", multi_config, logger=self.__logger
        )
        return [GetRuntimeRecommendsResult(*data) for data in ret]

    @logger_decorator
    def get_recipe_inherits(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetRecipeInheritsResult]:
        """Get recipes and its inherit recipes

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetRecipeInheritsResult]: See GetRecipeInheritsResult
        """
        ret: Mapping[str, List[str]] = self.__run_command(  # type: ignore
            self.__server_connection, "getRecipeInherits", multi_config, logger=self.__logger
        )
        return [GetRecipeInheritsResult(key, value) for key, value in ret.items()]

    @logger_decorator
    def get_bb_file_priority(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetBbFilePriorityResult]:
        """Get recipe files and its priority.

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetBbFilePriorityResult]: See GetBbFilePriorityResult
        """
        ret: Mapping[str, int] = self.__run_command(  # type: ignore
            self.__server_connection, "getBbFilePriority", multi_config, logger=self.__logger
        )
        return [GetBbFilePriorityResult(key, value) for key, value in ret.items()]

    @logger_decorator
    def get_default_preference(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetDefaultPreferenceResult]:
        """Get recipes and default preference.

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetDefaultPreference]: See GetDefaultPreference
        """
        ret: Mapping[str, int] = self.__run_command(  # type: ignore
            self.__server_connection, "getDefaultPreference", multi_config, logger=self.__logger
        )
        return [GetDefaultPreferenceResult(key, value) for key, value in ret.items()]

    @logger_decorator
    def get_skipped_recipes(self: "BBClient") -> List[GetSkippedRecipesResult]:
        """Get skipped recipes and its reasons, provides, alias

        Args:
            self (BBClient): none

        Returns:
            List[GetSkippedRecipesResult]: See GetSkippedRecipesResult
        """
        ret: List[List[Any]] = self.__run_command(self.__server_connection, "getSkippedRecipes", logger=self.__logger)  # type: ignore
        return [GetSkippedRecipesResult(*i) for i in ret]

    @logger_decorator
    def get_overlayed_recipes(self: "BBClient", multi_config: str = ""):
        return self.__run_command(
            self.__server_connection, "getOverlayedRecipes", multi_config, logger=self.__logger
        )

    @logger_decorator
    def get_file_appends(
        self: "BBClient", file_path: str, multi_config: str = ""
    ) -> List[str]:
        """Get append files

        Args:
            self (BBClient): none
            file_path (str): recipe file path
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[str]: append files
        """
        return self.__run_command(  # type: ignore
            self.__server_connection, "getFileAppends", file_path, multi_config, logger=self.__logger
        )

    @logger_decorator
    def get_all_appends(
        self: "BBClient", multi_config: str = ""
    ) -> List[GetAllAppendsResult]:
        """Get all append recipes

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[GetAllAppendsResult]: See GetAllAppendsResult
        """
        ret: List[List[str]] = self.__run_command(  # type: ignore
            self.__server_connection, "getAllAppends", multi_config, logger=self.__logger
        )
        return [GetAllAppendsResult(*i) for i in ret]

    @logger_decorator
    def find_providers(
        self: "BBClient", multi_config: str = ""
    ) -> List[FindProvidersResult]:
        """Get latest packages versions, prefered package versions, and whether there is an REQUIRED_VERSION

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[FindProvidersResult]: See FindProvidersResult
        """
        result: Any = self.__run_command(  # type: ignore
            self.__server_connection, "findProviders", multi_config, logger=self.__logger
        )
        ret: List[FindProvidersResult] = []
        for package in result[0].keys():
            ret.append(
                FindProvidersResult(
                    # yocto dunfell doesn't support result[3](required or not), so checking len(result) == 3
                    package, result[0][package], result[1][package], result[2][package] if len(result) == 3 else None 
                )
            )
        return ret

    @logger_decorator
    def find_best_provider(
        self: "BBClient", package_name: str, multi_config: str = ""
    ) -> List[str]:
        """Get best provider infos

        Args:
            self (BBClient): none
            package_name (str): a package name you want to know the detail of best provider
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[str]: Now under investigation.

        Note:
            | Return value is like [None, None, None, '/PATH/TO/RECIPE/gcc_11.3.bb']. Now under investigation.
        """
        # if you want to know the detail of this line, see bb.runqueue.split_mc.
        package_name = (
            "mc:" + multi_config + ":" + package_name if multi_config else package_name
        )
        return self.__run_command(  # type: ignore
            self.__server_connection, "findBestProvider", package_name, logger=self.__logger
        )

    @logger_decorator
    def all_providers(
        self: "BBClient", multi_config: str = ""
    ) -> List[AllProvidersResult]:
        """Get all providers versions and recipe file path

        Args:
            self (BBClient): none
            multi_config (str, optional): Defaults to "". See `here <https://docs.yoctoproject.org/dev-manual/common-tasks.html?highlight=multiconfigs#building-images-for-multiple-targets-using-multiple-configurations>`_

        Returns:
            List[AllProvidersResult]: See AllProvidersResult

        Note:
            | Return value is like below.
            | [
            |   ['nativesdk-go', [[['', '1.17.10', 'r0'], '/PATH/TO/RECIPE/go_1.17.10.bb']]],  # PN, [PE, PV, PR], recipe file path
            |   ['go', [[['', '1.17.10', 'r0'], '/PATH/TO/RECIPE/go_1.17.10.bb']]],
            | ]
        """
        ret: List[List[Any]] = self.__run_command(  # type: ignore
            self.__server_connection, "allProviders", multi_config, logger=self.__logger
        )
        return [AllProvidersResult(*i) for i in ret]

    @logger_decorator
    def get_runtime_providers(
        self: "BBClient", runtime_providers: List[str], multi_config: str = ""
    ):
        # TODO: What is runtime_provider
        return self.__run_command(
            self.__server_connection,
            "getRuntimeProviders",
            runtime_providers,
            multi_config,
            logger=self.__logger
        )

    @logger_decorator
    def data_store_connector_cmd(
        self: "BBClient", datastore_index: int, command: str, *args, **kwargs
    ) -> Any:
        """Data store management function

        Args:
            self (BBClient): none
            datastore_index (int): specify datastore_index. user can get this value by parse_recipe_file command.
            command (str): the method name of bb.data_smart.DataSmart
            args (Any): depends on command
            kwargs (Any): depends on command

        Returns:
            Any: command return

        Note:
            | User can input following commands. If you want to know detail of these, please see bb.data_smart.DataSmart.

            * createCopy
            * createCopy
            * delVar
            * delVarFlag
            * delVarFlags
            * disableTracking
            * enableTracking
            * expand
            * expandVarref
            * expandWithRefs
            * finalize
            * get
            * getVar
            * getVarFlag
            * getVarFlags
            * get_hash
            * hasOverrides
            * initVar
            * internal_finalize
            * items
            * keys
            * localkeys
            * need_overrides
            * pop
            * popitem
            * prependVar
            * prependVarFlag
            * renameVar
            * setVar
            * setVarFlag
            * setVarFlags
            * setdefault
            * update
            * values
        """
        return self.__run_command(
            self.__server_connection,
            "dataStoreConnectorCmd",
            datastore_index,
            command,
            args,
            kwargs,
            logger=self.__logger
        )

    @logger_decorator
    def data_store_connector_varhist_cmd(
        self: "BBClient", datastore_index: int, command: str, *args, **kwargs
    ) -> Any:
        """Data store variable history function

        Args:
            self (BBClient): none
            datastore_index (int): specify datastore_index. user can get this value by parse_recipe_file command.
            command (str): the method name of bb.data_smart.VariableHistory
            args (Any): depends on command
            kwargs (Any): depends on command

        Returns:
            Any: command return

        Note:
            | User can input following commands. If you want to know detail of these, please see bb.data_smart.VariableHistory.

            * copy
            * del_var_history
            * emit
            * get_variable_files
            * get_variable_items_files
            * get_variable_lines
            * get_variable_refs
            * record
            * rename_variable_hist
            * variable
        """
        return self.__run_command(
            self.__server_connection,
            "dataStoreConnectorVarHistCmd",
            datastore_index,
            command,
            args,
            kwargs,
            logger=self.__logger
        )

    @logger_decorator
    def data_store_connector_var_hist_cmd_emit(
        self: "BBClient",
        datastore_index: int,
        variable: str,
        comment: str,
        val: str,
        override_datastore_index: int,
    ) -> str:
        """Update variable in datastore by variable inoverride datastore

        Args:
            self (BBClient): none
            datastore_index (int): specify datastore_index. user can get this value by parse_recipe_file command.
            variable (str): varibale name
            comment (str): comment for update log()
            override_datastore_index (int): specify datastore_index. user can get this value by parse_recipe_file command.

        Returns:
            str: update log

        Note:
            To be investigate.
        """
        return self.__run_command(  # type: ignore
            self.__server_connection,
            "dataStoreConnectorVarHistCmdEmit",
            datastore_index,
            variable,
            comment,
            "",  # this parameter is not used at all, but accessed, so we have to set a value. If not so, then exception will occur.
            override_datastore_index,
            logger=self.__logger
        )

    @logger_decorator
    def data_store_connector_inc_hist_cmd(
        self: "BBClient", datastore_index: int, command: str, *args, **kwargs
    ) -> Any:
        """Data store include history function

        Args:
            self (BBClient): none
            datastore_index (int): specify datastore_index. user can get this value by parse_recipe_file command.
            command (str): the method name of bb.data_smart.IncludeHistory
            args (Any): depends on command
            kwargs (Any): depends on command

        Returns:
            Any: command return

        Note:
            | User can input following commands. If you want to know detail of these, please see bb.data_smart.IncludeHistory.

            * copy
            * include
            * emit
        """
        return self.__run_command(
            self.__server_connection,
            "dataStoreConnectorIncHistCmd",
            datastore_index,
            command,
            args,
            kwargs,
            logger=self.__logger
        )

    @logger_decorator
    def data_store_connector_release(self: "BBClient", datastore_index: int) -> None:
        """Discard data store

        Args:
            self (BBClient): none
            datastore_index (int): specify datastore_index. user can get this value by parse_recipe_file command.
        """
        self.__run_command(
            self.__server_connection, "dataStoreConnectorRelease", datastore_index, logger=self.__logger
        )

    @logger_decorator
    def parse_recipe_file(
        self: "BBClient",
        file_path: str,
        append: bool = True,
        append_list: Optional[str] = None,
        datastore_index: Optional[int] = None,
    ) -> Optional[int]:
        """Parse recipe file

        Args:
            self (BBClient): none
            file_path (str): recipe file path
            append (bool, optional): whether to append. Defaults to True.
            append_list (Optional[str], optional): Append file list. Defaults to None. If None, bitbake loads append files automatically.
            datastore_index (int): specify datastore_index. user can get this value by parse_recipe_file command.

        Returns:
            Optional[int]: data store index

        Note:
            | This commands parses recipe file and store result into datastore. User can access the result by data store index. It's not clear how the fifth parameter(datastore_index) works.
        """
        ret: Mapping[str, int] = (
            self.__run_command(  # type: ignore
                self.__server_connection,
                "parseRecipeFile",
                file_path,
                append,
                append_list,
                datastore_index,
                logger=self.__logger
            )
            if datastore_index
            else self.__run_command(
                self.__server_connection,
                "parseRecipeFile",
                file_path,
                append,
                append_list,
                logger=self.__logger
            )
        )

        return ret.dsindex if hasattr(ret, "dsindex") else ret["dsindex"] if ret else None

    # --- bitbake server async functions  ---
    @logger_decorator
    def build_file(
        self: "BBClient", file_path: str, task_name: str, internal: bool = False
    ) -> None:
        """Build recipe file

        This command will send following events. If you want to wait done, please use wait_done_async.


        * bb.event.BuildInit
        * bb.event.RecipePreFinalise
        * bb.event.RecipePostKeyExpansion
        * bb.event.RecipeTaskPreProcess
        * bb.event.RecipeParsed
        * bb.event.BuildStarted
        * bb.event.ProcessStarted
        * bb.event.ProcessProgress
        * bb.event.ProcessFinished
        * bb.runqueue.runQueueTaskStarted
        * bb.runqueue.runQueueTaskCompleted
        * bb.event.BuildCompleted
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            file_path (str): target recipe file path
            task_name (str): task name which will run
            internal (bool, optional): If True, bitbake will fire events that notify BuildStarted and BuildCompleted. Defaults to False.

        Note:
            | If you want to monitor BuildStarted and BuildCompleted event, use get_event.
        """
        self.__run_command(
            self.__server_connection, "buildFile", file_path, task_name, internal, logger=self.__logger
        )

    @logger_decorator
    def build_targets(
        self: "BBClient",
        targets: List[str],
        task_name: str,
    ) -> None:
        """Build package

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.runqueue.runQueueTaskStarted
        * bb.build.TaskStarted
        * bb.build.TaskProgress
        * bb.build.TaskSucceeded
        * bb.event.ProcessStarted
        * bb.event.ProcessProgress
        * bb.event.ProcessFinished
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            targets (List[str]): see Note section
            task_name (str): task name which will run

        Note:
            | User can input targets as follows. Please note that if you want to specify task by targets, you have to write do_xxx, not only xxx.
            | [
            |   "gcc",                              # only package name
            |   "mc:xxx_config:alsa",               # multiconfig and package name
            |   "multiconfig:yyy_config:vim",       # multiconfig and package name
            |   "mc:\*:clang"                       # all multiconfig and package name
            |   "mc:yyy_config:python:do_patch",    # multiconfig, package name and task name
            | ]
        """

        self.__run_command(self.__server_connection, "buildTargets", targets, task_name, logger=self.__logger)

    @logger_decorator
    def generate_dep_tree_event(
        self: "BBClient", targets: List[str], task_name: str
    ) -> None:
        """Request dependency tree information

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.TreeDataPreparationStarted
        * bb.event.TreeDataPreparationProgress
        * bb.event.TreeDataPreparationCompleted
        * bb.event.DepTreeGenerated
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            targets(List[str]): targets info. see Note section.
            task_name (str): task name. e.g.) do_build, do_fetch, etc...

        Note:
            | User can input targets as follows.
            | [
            |   "gcc",                              # only package name
            |   "mc:xxx_config:alsa",               # multiconfig and package name
            |   "multiconfig:yyy_config:vim",       # multiconfig and package name
            |   "mc:\*:clang"                        # all multiconfig and package name
            |   "mc:yyy_config:python:do_patch",    # multiconfig, package name and task name
            | ]
        """
        self.__run_command(
            self.__server_connection,
            "generateDepTreeEvent",
            targets,
            task_name,
            logger=self.__logger
        )

    @logger_decorator
    def generate_dot_graph(
        self: "BBClient", targets: List[str], task_name: str
    ) -> None:
        """Generate task dependency graph(task-depends.dot)

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.TreeDataPreparationStarted
        * bb.event.TreeDataPreparationProgress
        * bb.event.TreeDataPreparationCompleted
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            targets (List[str]): targets info. see Note section.
            task_name (str): task name. e.g.) do_build, do_fetch, etc...

        Note:
            | User can input targets as follows.
            | [
            |   "gcc",                              # only package name
            |   "mc:xxx_config:alsa",               # multiconfig and package name
            |   "multiconfig:yyy_config:vim",       # multiconfig and package name
            |   "mc:\*:clang"                        # all multiconfig and package name
            |   "mc:yyy_config:python:do_patch",    # multiconfig, package name and task name
            | ]
        """
        self.__run_command(
            self.__server_connection,
            "generateDotGraph",
            targets,
            task_name,
            logger=self.__logger
        )

    @logger_decorator
    def generate_targets_tree(
        self: "BBClient", bb_klass_file_path: str, package_names: List[str]
    ) -> None:
        """Generate target tree

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.TreeDataPreparationStarted
        * bb.event.TreeDataPreparationProgress
        * bb.event.TreeDataPreparationCompleted
        * bb.event.TargetsTreeGenerated
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            bb_klass_file_path (str): bbclass file path
            package_names (List[str]): target package names

        Note:
            | Use can receive result by bb.event.TargetsTreeGenerated event.
            | If you specify bb_klass_file_path, bitbake will add the packages that inherits bb_klass_file_path to package_names. If you don't want to do it, please input None to bb_klass_file_path.
        """
        self.__run_command(
            self.__server_connection,
            "generateTargetsTree",
            bb_klass_file_path,
            package_names,
            logger=self.__logger
        )

    @logger_decorator
    def find_config_files(self: "BBClient", variable_name: str) -> None:
        """Find Config files that define specified variable.

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.ConfigFilesFound
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            variable_name (str): _description_

        Note:
            | User can receive result by bb.event.ConfigFilesFound event.
        """
        self.__run_command(self.__server_connection, "findConfigFiles", variable_name, logger=self.__logger)

    @logger_decorator
    def find_files_matching_in_dir(
        self: "BBClient", target_file_name_substring: str, directory: str
    ) -> None:
        """Find files that matches the regex_pattern from the directory.

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.FilesMatchingFound
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            target_file_name_substring (str): Substrings of target file name. e.g.) ".conf", "xxx.bbappe", etc...
            directory (str): Target directory. Base directory is ${BBPATH}.

        Note:
            | Use can receive result by bb.event.FilesMatchingFound event.
        """
        self.__run_command(
            self.__server_connection,
            "findFilesMatchingInDir",
            target_file_name_substring,
            directory,
            logger=self.__logger
        )

    @logger_decorator
    def test_cooker_command_event(self: "BBClient", pattern: str) -> None:
        """Dummy command

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.FilesMatchingFound
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            pattern (str): dummy param
        """
        self.__run_command(self.__server_connection, "testCookerCommandEvent", pattern, logger=self.__logger)

    @logger_decorator
    def find_config_file_path(self: "BBClient", config_file_name: str) -> None:
        """Find config file path

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.ConfigFilePathFound
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            config_file_name (str): target config file name

        Note:
            | Use can receive result by bb.event.ConfigFilePathFound event.
        """
        self.__run_command(
            self.__server_connection, "findConfigFilePath", config_file_name, logger=self.__logger
        )

    @logger_decorator
    def show_versions(self: "BBClient") -> None:
        """Show all packages versions

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none

        Note:
            | bbclient doesn't display any information. If you want to use this feature, please use bitbake-layers.
        """
        self.__run_command(self.__server_connection, "showVersions", logger=self.__logger)

    @logger_decorator
    def show_environment_target(self: "BBClient", package_name: str = "") -> None:
        """Show variables for specified package

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.ConfigParsed
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            package_name (str): target package name

        Note:
            | bbclient doesn't display any information. If you want to use this feature, please use bitbake-gervar or bitbake -e.
        """
        self.__run_command(
            self.__server_connection, "showEnvironmentTarget", package_name, logger=self.__logger
        )

    @logger_decorator
    def show_environment(self: "BBClient", bb_file_path: str) -> None:
        """Show variables for specified recipe

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.ConfigParsed
        * bb.event.CacheLoadStarted
        * bb.event.CacheLoadProgress
        * bb.event.CacheLoadCompleted
        * bb.event.RecipePreFinalise
        * bb.event.RecipePostKeyExpansion
        * bb.event.RecipeTaskPreProcess
        * bb.event.RecipeParsed
        * bb.event.ConfigParsed
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
            bb_file_path (str): target recipe path

        Note:
            | bbclient doesn't display any information. If you want to use this feature, please use bitbake-gervar or bitbake -e.
        """
        self.__run_command(self.__server_connection, "showEnvironment", bb_file_path, logger=self.__logger)

    @logger_decorator
    def parse_files(self: "BBClient") -> None:
        """Parse all bb files.

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.ReachableStamps
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
        """
        self.__run_command(self.__server_connection, "parseFiles", logger=self.__logger)

    @logger_decorator
    def compare_revisions(self: "BBClient") -> None:
        """Exit async command

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none

        Note:
            | TODO: investigate the detail.
            | This determines if the cache is out of date, and if so, this terminates asynchronous processing.
        """
        self.__run_command(self.__server_connection, "compareRevisions", logger=self.__logger)

    @logger_decorator
    def trigger_event(self: "BBClient", evene_name: str) -> None:
        """Send event

        Args:
            self (BBClient): none
            evene_name (str): event class name.

        Note:
            | Send evene_name event. User can receive this event by get_event.
        """
        self.__run_command(self.__server_connection, "triggerEvent", evene_name, logger=self.__logger)

    @logger_decorator
    def reset_cooker(self: "BBClient") -> None:
        """Reset cooker state and caches.

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.event.ConfigParsed
        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none

        Note:
            | TODO: investigate the detail.
        """
        self.__run_command(self.__server_connection, "resetCooker", logger=self.__logger)

    @logger_decorator
    def client_complete(self: "BBClient") -> None:
        """Notify client will be close

        This command will send following events. If you want to wait done, please use wait_done_async.

        * bb.command.CommandCompleted
        * bb.command.CommandFailed
        * bb.command.CommandExit

        Args:
            self (BBClient): none
        """
        self.__run_command(self.__server_connection, "clientComplete", logger=self.__logger)

    @logger_decorator
    def find_sigInfo(
        self: "BBClient",
        package_name_with_multi_config: str,
        task_name: str,
        sigs: List[str],
    ) -> None:
        """Find signature info files via the signature generator(?)

        Args:
            self (BBClient): none
            package_name_with_multi_config (str): TODO
            task_name (str): TODO
            sigs (List[str]): TODO

        Note:
            | Use can receive result by bb.event.FindSigInfoResult event.
        """
        self.__run_command(
            self.__server_connection,
            "findSigInfo",
            package_name_with_multi_config,
            task_name,
            sigs,
            logger=self.__logger
        )

    # --- private functions ---

    @staticmethod
    def __connect_server(
        project_path: str
    ) -> Tuple["bb.server.xmlrpcclient.BitBakeXMLRPCServerConnection", "module"]:  # type: ignore
        """Connect to server

        Returns:
            _type_: ("bb.server.xmlrpcclient.BitBakeXMLRPCServerConnection", "module")
        """
        # TODO: use shell not to be depends on bb modules
        sys.path.append(f"{project_path}/bitbake/lib")
        from bb.main import setup_bitbake, BitBakeConfigParameters  # type: ignore
        from bb.tinfoil import TinfoilConfigParameters  # type: ignore
        from bb.cookerdata import CookerConfiguration  # type: ignore

        config_params: TinfoilConfigParameters = TinfoilConfigParameters(
            config_only=False, quiet=4
        )
        try:
            # dunfell or more older
            cookerconfig = CookerConfiguration()
            cookerconfig.setConfigParameters(config_params)
            server_connection, ui_module = setup_bitbake(config_params, cookerconfig)
        except:
            # kirkstone or more newer
            server_connection, ui_module = setup_bitbake(config_params)
        ui_module.main(
            server_connection.connection, server_connection.events, config_params
        )
        return server_connection, ui_module

    @staticmethod
    def __run_command(server_connection, command: str, *params: Any, logger: Optional[Logger]) -> Optional[Any]:
        """Run command

        Args:
            server_connection (_type_): use return value of __connect_server()
            command (str): commands bitbake defined
            params (Any): paramters for command
            logger (Optional[Logger]): logger for debugging

        Returns:
            Optional[Any]: command return
        """
        commandline: List[str] = [command]
        commandline.extend(params if params else [])
        try:
            result = server_connection.connection.runCommand(commandline)
        except:
            if logger:
                logger.error(f"{command} failed beacuse {result}.")
            return None
        return result[0]



from .common import *
from bbclient import *

params = [
    ("MACHINE", [ConfigFilesFoundEvent], CommandCompletedEvent),
    ("DUMMY", [], CommandCompletedEvent),
]

@pytest.mark.parametrize("variable_name, except_event_types, expect_command_result", params)
def test_find_config_files_main(main_client: BBClient, variable_name: str, except_event_types: List[BBEventBase], expect_command_result: BBEventBase) -> None:
    __test_impl(main_client, variable_name, except_event_types, expect_command_result)

@pytest.mark.parametrize("variable_name, except_event_types, expect_command_result", params)
def test_find_config_files_kirkstone(kirkstone_client: BBClient, variable_name: str, except_event_types: List[BBEventBase], expect_command_result: BBEventBase) -> None:
    __test_impl(kirkstone_client, variable_name, except_event_types, expect_command_result)

@pytest.mark.parametrize("variable_name, except_event_types, expect_command_result", params)
def test_find_config_files_dunfell(dunfell_client: BBClient, variable_name: str, except_event_types: List[BBEventBase], expect_command_result: BBEventBase) -> None:
    __test_impl(dunfell_client, variable_name, except_event_types, expect_command_result)

def __test_impl(client: BBClient, variable_name: str, except_event_types: List[BBEventBase], expect_command_result: BBEventBase) -> None:
    client.find_config_files(variable_name)
    for expect_event_type in except_event_types:
        ret: Optional[BBEventBase] = client.wait_event([expect_event_type], 3)
        assert isinstance(ret, expect_event_type)
    ret: BBEventBase = client.wait_done_async()
    assert isinstance(ret, expect_command_result)


from .common import *
from bbclient import *

params = [
    ("bblayers.conf"),
]

@pytest.mark.parametrize("config_file_name", params)
def test_find_config_file_path_main(main_client: BBClient, config_file_name: str) -> None:
    __test_impl(main_client, config_file_name)

@pytest.mark.parametrize("config_file_name", params)
def test_find_config_file_path_kirkstone(kirkstone_client: BBClient, config_file_name: str) -> None:
    __test_impl(kirkstone_client, config_file_name)

@pytest.mark.parametrize("config_file_name", params)
def test_find_config_file_path_dunfell(dunfell_client: BBClient, config_file_name: str) -> None:
    __test_impl(dunfell_client, config_file_name)

def __test_impl(client: BBClient, config_file_name: str) -> None:
    client.find_config_file_path(config_file_name)
    ret: Optional[BBEventBase] = client.wait_event([ConfigFilePathFoundEvent], 3)
    assert isinstance(ret, ConfigFilePathFoundEvent)
    ret: ConfigFilePathFoundEvent
    assert os.path.isfile(ret.path)
    ret: Optional[BBEventBase] = client.wait_done_async()
    assert isinstance(ret, CommandCompletedEvent)


from .common import *
from bbclient import *

def test_get_uihandler_num_main(main_client: BBClient) -> None:
    __test_impl(main_client)

def test_get_uihandler_num_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_get_uihandler_num_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    variable_name: str = "MACHINE"
    ret: str = client.get_variable(variable_name)
    assert ret is not None


#!/usr/bin/python3

from bbclient import *

import os
import logging
import pytest
from logging import Logger, StreamHandler, getLogger


CUR_FILE_PATH: str = os.path.dirname(__file__)
INIT_COMMAND: str = ". oe-init-build-env"

@pytest.fixture(scope="module")
def main_client():
    PATH_TO_MAIN = CUR_FILE_PATH + "/../main"
    client: BBClient = BBClient(PATH_TO_MAIN, INIT_COMMAND, None)
    client.start_server()
    ui_handler: int = client.get_uihandler_num()
    client.set_event_mask(ui_handler, logging.WARNING, {}, ["*"])
    client.parse_files()
    client.wait_done_async()
    yield client
    client.stop_server()

@pytest.fixture(scope="module")
def kirkstone_client():
    PATH_TO_KIRKSTONE = CUR_FILE_PATH + "/../kirkstone"
    client: BBClient = BBClient(PATH_TO_KIRKSTONE, INIT_COMMAND, None)
    client.start_server()
    ui_handler: int = client.get_uihandler_num()
    client.set_event_mask(ui_handler, logging.WARNING, {}, ["*"])
    client.parse_files()
    client.wait_done_async()
    yield client
    client.stop_server()

@pytest.fixture(scope="module")
def dunfell_client():
    PATH_TO_DUNFELL = CUR_FILE_PATH + "/../dunfell"
    client: BBClient = BBClient(PATH_TO_DUNFELL, INIT_COMMAND, None)
    client.start_server()
    ui_handler: int = client.get_uihandler_num()
    client.set_event_mask(ui_handler, logging.WARNING, {}, ["*"])
    client.parse_files()
    client.wait_done_async()
    yield client
    client.stop_server()

def setup_logger() -> Logger:
    ch = StreamHandler()
    ch.setLevel(logging.DEBUG)
    formatter = logging.Formatter('[%(name)s][%(asctime)s][%(levelname)s]: %(message)s')
    ch.setFormatter(formatter)
    logger: Logger = getLogger("bbclient")
    logger.setLevel('DEBUG')
    logger.addHandler(ch)
    return logger



from .common import * 
from bbclient import *

def test_all_providers_main(main_client: BBClient) -> None:
    __test_imple(main_client)

def test_all_providers_kirkstone(kirkstone_client: BBClient) -> None:
    __test_imple(kirkstone_client)

def test_all_providers_dunfell(dunfell_client: BBClient) -> None:
    __test_imple(dunfell_client)

def __test_imple(client: BBClient) -> None:
    ret: List[AllProvidersResult] = client.all_providers()
    assert ret is not None


from distutils.cmd import Command
from .common import *
from bbclient import *

def test_parse_files_main(main_client: BBClient) -> None:
    __test_impl(main_client)
    
def test_parse_files_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_parse_files_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    client.parse_files()
    ret: Optional[BBEventBase] = client.wait_event([ReachableStampsEvent])
    assert isinstance(ret, ReachableStampsEvent)
    ret: Optional[BBEventBase] = client.wait_done_async()
    assert isinstance(ret, CommandCompletedEvent)


from .common import *
from bbclient import *

params = [
    ("meta/classes/core-image.bbclass", ["gcc"]),
    ("meta/classes/core-image.bbclass", ["busybox", "python3:do_fetch"]),
]

@pytest.mark.parametrize("klass_file_path, targets", params)
def test_generate_targets_tree_main(main_client: BBClient, klass_file_path: str, targets: List[str]) -> None:
    __test_impl(main_client, klass_file_path, targets)

@pytest.mark.parametrize("klass_file_path, targets", params)
def test_generate_targets_tree_kirkstone(kirkstone_client: BBClient, klass_file_path: str, targets: List[str]) -> None:
    __test_impl(kirkstone_client, klass_file_path, targets)

@pytest.mark.parametrize("klass_file_path, targets", params)
def test_generate_targets_tree_dunfell(dunfell_client: BBClient, klass_file_path: str, targets: List[str]) -> None:
    __test_impl(dunfell_client, klass_file_path, targets)

def __test_impl(client: BBClient, klass_file_path: str, targets: List[str]) -> None:
    client.generate_targets_tree(klass_file_path, targets)
    ret: Optional[BBEventBase] = client.wait_event([TreeDataPreparationStartedEvent])
    assert isinstance(ret, TreeDataPreparationStartedEvent)
    ret: Optional[BBEventBase] = client.wait_event([TreeDataPreparationProgressEvent])
    assert isinstance(ret, TreeDataPreparationProgressEvent)
    ret: Optional[BBEventBase] = client.wait_event([TreeDataPreparationCompletedEvent])
    assert isinstance(ret, TreeDataPreparationCompletedEvent)
    ret: Optional[BBEventBase] = client.wait_event([TargetsTreeGeneratedEvent])
    assert isinstance(ret, TargetsTreeGeneratedEvent)
    ret: Optional[BBEventBase] = client.wait_done_async()
    assert isinstance(ret, CommandCompletedEvent)


from .common import *
from bbclient import *

def test_find_files_matching_in_dir_main(main_client: BBClient) -> None:
    __test_impl(main_client)
    
def test_find_config_files_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_find_config_files_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    ret: List[FindProvidersResult] = client.find_providers()
    assert ret is not None
    assert len(ret) != 0



from .common import * 
from bbclient import *

params = [
    ("busybox", "fetch", CommandCompletedEvent),
    ("dummy_package", "fetch", CommandFailedEvent),
]

@pytest.mark.parametrize("package, task, expect", params)
def test_build_file_main(main_client: BBClient, package: str, task: str, expect: Type[BBEventBase]) -> None:
    __test_imple(main_client, package, task, expect)

@pytest.mark.parametrize("package, task, expect", params)
def test_build_file_kirkstone(kirkstone_client: BBClient, package: str, task: str, expect: Type[BBEventBase]) -> None:
    __test_imple(kirkstone_client, package, task, expect)

@pytest.mark.parametrize("package, task, expect", params)
def test_build_file_dunfell(dunfell_client: BBClient, package: str, task: str, expect: Type[BBEventBase]) -> None:
    __test_imple(dunfell_client, package, task, expect)

def __test_imple(client: BBClient, package: str, task: str, expect: Type[BBEventBase]) -> None:
    best_provider: List[str] = client.find_best_provider(package)
    recipe_file_path: str = best_provider[3]
    client.build_file(recipe_file_path, task)
    result: Optional[BBEventBase] = client.wait_done_async()
    assert isinstance(result, expect)


""" 
This file provides definition for events from bitbake server
"""

from typing import Mapping, Any, List, Type

class BBEventBase:
    """Base class for all the events

    Attributes:
        event_name (str): event name like "bb.build.TaskProgress"
    """
    EVENT_NAME: str = "bb.build.TaskFailed"

    def __init__(self: "BBEventBase", event_name: str) -> None:
        """init

        Args:
            self (BBEventBase): none
            event_name (str): event name like "bb.build.TaskProgress"
        """
        self.event_name: str = event_name

    @classmethod
    def is_target(cls, event_name: str) -> bool:
        """Determine if the event is a target event.

        Args:
            event_name (str): event name like "bb.build.TaskProgress"

        Returns:
            bool: whether the event is a target or not
        """
        return cls.EVENT_NAME == event_name

class TaskFailedEvent(BBEventBase):
    EVENT_NAME: str = "bb.build.TaskFailed"

    def __init__(self: "TaskFailedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        # TODO

class TaskProgressEvent(BBEventBase):
    EVENT_NAME: str = "bb.build.TaskProgress"

    def __init__(self: "TaskProgressEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.progress: int = data["progress"]
        self.rate: str = data["rate"]

class TaskStartedEvent(BBEventBase):
    EVENT_NAME: str = "bb.build.TaskStarted"

    def __init__(self: "TaskStartedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.task: str = data["_task"]
        self.fn: str = data["_fn"]
        self.package: str = data["_package"]
        self.mc: str = data["_mc"]
        self.taskfile: str = data["taskfile"]
        self.taskname: str = data["taskname"]
        self.logfile: str = data["logfile"]
        self.time: float = data["time"]
        self.pn: str = data["pn"]
        self.pv: str = data["pv"]
        self.message: str = data["_message"]
        self.taskflags: Any = data["taskflags"]

class TaskSucceededEvent(BBEventBase):
    EVENT_NAME: str = "bb.build.TaskSucceeded"

    def __init__(self: "TaskSucceededEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.task: str = data["_task"]
        self.fn: str = data["_fn"]
        self.package: str = data["_package"]
        self.mc: str = data["_mc"]
        self.taskfile: str = data["taskfile"]
        self.taskname: str = data["taskname"]
        self.logfile: str = data["logfile"]
        self.time: float = data["time"]
        self.pn: str = data["pn"]
        self.pv: str = data["pv"]
        self.message: str = data["_message"]

class CommandCompletedEvent(BBEventBase):
    EVENT_NAME: str = "bb.command.CommandCompleted"

    def __init__(self: "CommandCompletedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]

class CommandFailedEvent(BBEventBase):
    EVENT_NAME: str = "bb.command.CommandFailed"

    def __init__(self: "CommandFailedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.error: str = data["error"]
        self.exitcode: str = data["exitcode"]

class CommandExitEvent(BBEventBase):
    EVENT_NAME: str = "bb.command.CommandExit"

    def __init__(self: "CommandExitEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        # TODO

class CacheLoadCompletedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.CacheLoadCompleted"

    def __init__(self: "CacheLoadCompletedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.total: int = data["total"]
        self.msg: str = data["msg"]
        self.num_entries: int = data["num_entries"]

class CacheLoadProgressEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.CacheLoadProgress"

    def __init__(self: "CacheLoadProgressEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.current: int = data["current"]
        self.total: int = data["total"]
        self.msg: str = data["msg"]

class CacheLoadStartedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.CacheLoadStarted"

    def __init__(self: "CacheLoadStartedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: str = data["pid"]
        self.msg: str = data["msg"]
        self.total: int = data["total"]

class ConfigFilePathFoundEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.ConfigFilePathFound"

    def __init__(self: "ConfigFilePathFoundEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.path: str = data["_path"]

class ConfigFilesFoundEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.ConfigFilesFound"

    def __init__(self: "ConfigFilesFoundEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.variable: str = data["_variable"]
        self.values: List[str] = data["_values"]

class ConfigParsedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.ConfigParsed"

    def __init__(self: "ConfigParsedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]

class DepTreeGeneratedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.DepTreeGenerated"

    def __init__(self: "DepTreeGeneratedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        # TODO: _depgraph has complex content.
        self.depgraph : Mapping[str, Any] = data["_depgraph"]

class FilesMatchingFoundEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.FilesMatchingFound"

    def __init__(self: "FilesMatchingFoundEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.pattern: str = data["_pattern"]
        self.matches: List[str] = data["_matches"]

class ProcessFinishedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.ProcessFinished"

    def __init__(self: "ProcessFinishedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.processname: str = data["processname"]

class ProcessProgressEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.ProcessProgress"

    def __init__(self: "ProcessProgressEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.processname: int = data["processname"]
        self.progress: float = data["progress"]

class ProcessStartedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.ProcessStarted"

    def __init__(self: "ProcessStartedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.processname: int = data["processname"]
        self.total: int = data["total"]

class ReachableStampsEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.ReachableStamps"

    def __init__(self: "ReachableStampsEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        # TODO: define more detail
        self.stamps: Mapping[str, str] = data["stamps"]

class RecipeParsedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.RecipeParsed"

    def __init__(self: "RecipeParsedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.fn: str = data["fn"]

class RecipePostKeyExpansionEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.RecipePostKeyExpansion"

    def __init__(self: "RecipePostKeyExpansionEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.fn: str = data["fn"]

class RecipePreFinaliseEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.RecipePreFinalise"

    def __init__(self: "RecipePreFinaliseEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.fn: str = data["fn"]

class RecipeTaskPreProcessEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.RecipeTaskPreProcess"

    def __init__(self: "RecipeTaskPreProcessEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.fn: str = data["fn"]
        self.tasklist: List[str] = data["tasklist"]

class TargetsTreeGeneratedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.TargetsTreeGenerated"

    def __init__(self: "TargetsTreeGeneratedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.model: Mapping[str, Any] = data["_model"]

class TreeDataPreparationCompletedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.TreeDataPreparationCompleted"

    def __init__(self: "TreeDataPreparationCompletedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.total: int = data["total"]
        self.msg: str = data["msg"]

class TreeDataPreparationProgressEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.TreeDataPreparationProgress"

    def __init__(self: "TreeDataPreparationProgressEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        {'pid': 0, 'current': 1, 'total': 1, 'msg': 'Preparing tree data: 1/1'}
        self.pid: int = data["pid"]
        self.current: int = data["current"]
        self.total: int = data["total"]
        self.msg: str = data["msg"]

class TreeDataPreparationStartedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.TreeDataPreparationStarted"

    def __init__(self: "TreeDataPreparationStartedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.msg: str = data["msg"]

class runQueueTaskFailedEvent(BBEventBase):
    EVENT_NAME: str = "bb.runqueue.runQueueTaskFailed"

    def __init__(self: "runQueueTaskFailedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        # TODO

class runQueueTaskStartedEvent(BBEventBase):
    EVENT_NAME: str = "bb.runqueue.runQueueTaskStarted"

    def __init__(self: "runQueueTaskStartedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.taskid: str = data["taskid"]
        self.taskstring: str = data["taskstring"]
        self.taskname: str = data["taskname"]
        self.taskfile: str = data["taskfile"]
        self.taskhash: str = data["taskhash"]
        self.stats: Any = data["stats"].__dict__
        self.noexec: bool = data["noexec"]

class runQueueTaskSkippedEvent(BBEventBase):
    EVENT_NAME: str = "bb.runqueue.runQueueTaskSkipped"

    def __init__(self: "runQueueTaskSkippedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.taskid: str = data["taskid"]
        self.taskstring: str = data["taskstring"]
        self.taskname: str = data["taskname"]
        self.taskfile: str = data["taskfile"]
        self.taskhash: str = data["taskhash"]
        self.stats: Any = data["stats"]
        self.reason: str = data["reason"]

class sceneQueueCompleteEvent(BBEventBase):
    EVENT_NAME: str = "bb.runqueue.sceneQueueComplete"

    def __init__(self: "sceneQueueCompleteEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.stats: Any = data["stats"]

class runQueueTaskCompletedEvent(BBEventBase):
    EVENT_NAME: str = "bb.runqueue.runQueueTaskCompleted"

    def __init__(self: "runQueueTaskCompletedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.taskid: str = data["taskid"]
        self.taskstring: str = data["taskstring"]
        self.taskname: str = data["taskname"]
        self.taskfile: str = data["taskfile"]
        self.taskhash: str = data["taskhash"]
        self.stats: Any = data["stats"]

class BuildInitEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.BuildInit"

    def __init__(self: "BuildInitEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.name: str = data["_name"]
        self.pkgs: List[Any] = data["_pkgs"]
        self.pid: int = data["pid"]
        self.failures: int = data["_failures"]

class BuildStartedEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.BuildStarted"

    def __init__(self: "BuildStartedEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.msg: str = data["msg"]
        self.name: str = data["_name"]
        self.pkgs: List[str] = data["_pkgs"]
        self.failures: int = data["_failures"]

class HeartbeatEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.HeartbeatEvent"

    def __init__(self: "HeartbeatEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.time: float = data["time"]

class NoProviderEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.NoProvider"

    def __init__(self: "NoProviderEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.item: int = data["_item"]
        self.runtime: bool = data["_runtime"]
        self.dependees: List[Any] = data["_dependees"]
        self.reasons: List[Any] = data["_reasons"]
        self.close_matches: List[Any] = data["_close_matches"]
        

class MonitorDiskEventEvent(BBEventBase):
    EVENT_NAME: str = "bb.event.MonitorDiskEvent"

    def __init__(self: "MonitorDiskEventEvent", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.pid: int = data["pid"]
        self.disk_usage: Mapping[str, Any] = data["disk_usage"]

class LogRecord(BBEventBase):
    EVENT_NAME: str = "logging.LogRecord"

    def __init__(self: "LogRecord", data: Mapping[str, Any]) -> None:
        super().__init__(self.EVENT_NAME)
        self.log: Mapping[str, Any] = data


class UnknownEvent(BBEventBase):

    def __init__(self: "UnknownEvent", event_name: str, data: Mapping[str, Any]) -> None:
        super().__init__(event_name)
        self.data: Mapping[str, Any] = data

ALL_BB_EVENTS: List[Type[BBEventBase]] = [
    TaskFailedEvent,
    TaskProgressEvent,
    TaskStartedEvent,
    TaskSucceededEvent,
    CommandCompletedEvent,
    CommandFailedEvent,
    CommandExitEvent,
    CacheLoadCompletedEvent,
    CacheLoadProgressEvent,
    CacheLoadStartedEvent,
    ConfigFilePathFoundEvent,
    ConfigFilesFoundEvent,
    ConfigParsedEvent,
    DepTreeGeneratedEvent,
    FilesMatchingFoundEvent,
    ProcessFinishedEvent,
    ProcessProgressEvent,
    ProcessStartedEvent,
    ReachableStampsEvent,
    RecipeParsedEvent,
    RecipePostKeyExpansionEvent,
    RecipePreFinaliseEvent,
    RecipeTaskPreProcessEvent,
    TargetsTreeGeneratedEvent,
    TreeDataPreparationCompletedEvent,
    TreeDataPreparationProgressEvent,
    TreeDataPreparationStartedEvent,
    runQueueTaskFailedEvent,
    runQueueTaskStartedEvent,
    runQueueTaskSkippedEvent,
    sceneQueueCompleteEvent,
    runQueueTaskCompletedEvent,
    BuildInitEvent,
    BuildStartedEvent,
    HeartbeatEvent,
    NoProviderEvent,
    MonitorDiskEventEvent,
    LogRecord,
]


from .common import *
from bbclient import *

def test_get_uihandler_num_main(main_client: BBClient) -> None:
    __test_impl(main_client)

def test_get_uihandler_num_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_get_uihandler_num_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    ret: str = client.match_file(".*/core-image-minimal.bb")
    # this assert will fail because of bitbake bug. See match_file docstring.
    #assert ret is not None 


from .common import *
from bbclient import *

def test_set_variable(main_client: BBClient) -> None:
    variable_name: str = "DUMMY"
    variable_value: str = "test"
    main_client.set_variable(variable_name, variable_value)
    ret: str = main_client.get_variable(variable_name)
    assert ret == variable_value


from .common import *
from bbclient import *

params = [
    ("gcc"),
]

@pytest.mark.parametrize("package_name", params)
def test_find_best_provider_main(main_client: BBClient, package_name) -> None:
    __test_imple(main_client, package_name)

@pytest.mark.parametrize("package_name", params)
def test_find_best_provider_kirkstone(kirkstone_client: BBClient, package_name) -> None:
    __test_imple(kirkstone_client, package_name)

@pytest.mark.parametrize("package_name", params)
def test_find_best_provider_dunfell(dunfell_client: BBClient, package_name) -> None:
    __test_imple(dunfell_client, package_name)

def __test_imple(client: BBClient, package_name) -> None:
    result: List[str] = client.find_best_provider(package_name)
    recipe_file_path: str = result[3]
    data_store_index: int = client.parse_recipe_file(recipe_file_path)
    package_name_from_recipe: str = client.data_store_connector_cmd(data_store_index, "getVar", "PN")
    assert package_name == package_name_from_recipe


from .common import *
from bbclient import *

def test_parse_configuration_main(main_client: BBClient) -> None:
    __test_impl(main_client)

def test_parse_configuration_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_parse_configuration_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    client.parse_configuration()
    # This command will return no results and events.


#!/usr/bin/env python3
""" 
This file provides common definition for ease of understanding in/out of commands
"""

import json
from typing import Mapping, Any, List, Optional, Union
from collections import namedtuple
from enum import Enum

VariableHistory = namedtuple(
    "VariableHistory",
    ["parsing", "variable", "file", "line", "op", "detail", "flag", "func"],
)


class getAllKeysWithFlagsResult:
    """getAllKeysWithFlagsResult

    Attributes:
        name (str): variable name
        value (str): variable value
        histories (List[VariableHistory]): see VariableHistory.
        flags (Mapping[str, str]): flag and value map
    """

    def __init__(
        self: "getAllKeysWithFlagsResult", name: str, data: Mapping[str, Any]
    ) -> None:
        self.name: str = name

        try:
            self.value: str = data["v"]
            self.histories: List[VariableHistory] = [
                self.__load_one_history(hist) for hist in data["history"]
            ]
            self.flags: Mapping[str, str] = {
                key: value
                for key, value in data.items()
                if key != "v" and key != "history"
            }
        except:
            raise Exception

    @staticmethod
    def __truncate(string, length):
        ellipsis = "..."
        return string[:length] + (ellipsis if string[length:] else "")

    @staticmethod
    def __load_one_history(hist_data: Mapping) -> VariableHistory:
        parsing = hist_data.get("parsing", None)
        variable = hist_data.get("variable", None)
        file = hist_data.get("file", None)
        line = hist_data.get("line", None)
        op = hist_data.get("op", None)
        detail = hist_data.get("detail", None)
        flag = hist_data.get("flag", None)
        func = hist_data.get("func ", None)
        error = {
            key: value
            for key, value in hist_data.items()
            if not key
            in ["parsing", "variable", "file", "line", "op", "detail", "flag", "func"]
        }
        if error:
            print(error)
        return VariableHistory(parsing, variable, file, line, op, detail, flag, func)


class BBFeature(Enum):
    HOB_EXTRA_CACHES = 0
    BASEDATASTORE_TRACKING = 1
    SEND_SANITYEVENTS = 2


class GetLayerPrioritiesResult:
    """getLayerPriorities Result

    Attributes:
        name (str): variable name
        path (str): variable value
        path2 (str): variable value
        priority (int): see VariableHistory.
    """

    def __init__(self: "GetLayerPrioritiesResult", data: List[Any]) -> None:
        self.name: str = data[0]
        self.path: str = data[1]
        self.path2: str = data[2]
        self.priority: int = data[3]


class GetRecipesResult:
    """getRecipes result

    Attributes:
        package_name (str): package name
        recipe_files (List[str]): recipe file paths
    """
    def __init__(self: "GetRecipesResult", data: List[Any]) -> None:
        self.package_name: str = data[0]
        self.recipe_files: List[str] = data[1]


class GetRecipeDependsResult:
    """getRecipeDepends result

    Attributes:
        recipe_file_path (str): recipe file path
        depend_package_names (List[str]): package names that the recipe file depends on
    """
    def __init__(self: "GetRecipeDependsResult", data: List[Any]) -> None:
        self.recipe_file_path: str = data[0]
        self.depend_package_names: List[str] = data[1]


class GetRecipeVersionsResult:
    """getRecipeVersions result

    Attributes:
        recipe_file_path (str): recipe file path
        pe (str): package epoch
        pv (str): package version
        pr (str): package revision
    """
    def __init__(
        self: "GetRecipeVersionsResult", version: List[str], recipe_file_path: str
    ):
        self.recipe_file_path: str = recipe_file_path
        self.pe: str = version[0]
        self.pv: str = version[1]
        self.pr: str = version[2]


class GetRecipeProvidesResult:
    """getRecipeProvides result

    Attributes:
        recipe_file_path (str): recipe file path
        packages (List[str]): package names that the recipe file provides
    """
    def __init__(
        self: "GetRecipeProvidesResult", recipe_file_path: str, packages: List[str]
    ):
        self.recipe_file_path: str = recipe_file_path
        self.packages: List[str] = packages


class GetRecipePackagesResult:
    """getRecipePackages result

    Attributes:
        package_name (str): package name 
        recipe_file_paths (List[str]): recipe file paths that provides the package
    """
    def __init__(
        self: "GetRecipePackagesResult", package_name: str, recipe_file_paths: List[str]
    ):
        self.package_name: str = package_name
        self.recipe_file_paths: List[str] = recipe_file_paths


class GetRecipePackagesDynamicResult:
    """getRecipePackagesDynamic result

    Attributes:
        dynamic_package_name (str): dynamic package name 
        recipe_file_paths (List[str]): recipe file paths that provides the dynamic package
    """
    def __init__(
        self: "GetRecipePackagesDynamicResult",
        dynamic_package_name: str,
        recipe_file_paths: List[str],
    ):
        self.dynamic_package_name: str = dynamic_package_name
        self.recipe_file_paths: List[str] = recipe_file_paths


class GetRProvidersResult:
    """getRProviders result

    Attributes:
        package_alias_name (str): package alias name 
        recipe_file_paths (List[str]): recipe file paths that provides the alias package 
    """
    def __init__(
        self: "GetRProvidersResult",
        package_alias_name: str,
        recipe_file_paths: List[str],
    ):
        self.package_alias_name: str = package_alias_name
        self.recipe_file_paths: List[str] = recipe_file_paths


class GetRuntimeDependsResult:
    """getRProviders result

    Attributes:
        recipe_file_path (str): recipe file path 
        package_depenedncy (Mapping[str, List[str]]): package names that the recipe file depends on
    """
    def __init__(
        self: "GetRuntimeDependsResult",
        recipe_file_path: str,
        package_depenedncy: Mapping[str, List[str]],
    ) -> None:
        self.recipe_file_path: str = recipe_file_path
        self.package_depenedncy: Mapping[str, List[str]] = package_depenedncy


class GetRecipeInheritsResult:
    """getRecipeInherits result

    Attributes:
        recipe_file_path (str): recipe file path 
        inherit_file_paths (List[str]): inherit recipe file paths that the recipe file inherits
    """
    def __init__(
        self: "GetRecipeInheritsResult",
        recipe_file_path: str,
        inherit_file_paths: List[str],
    ) -> None:
        self.recipe_file_path: str = recipe_file_path
        self.inherit_file_paths: List[str] = inherit_file_paths


class GetBbFilePriorityResult:
    """getBbFilePriority result

    Attributes:
        recipe_file_path (str): recipe file path 
        priority (int): priority of the recipe file
    """
    def __init__(
        self: "GetBbFilePriorityResult", recipe_file_path: str, priority: int
    ) -> None:
        self.recipe_file_path: str = recipe_file_path
        self.priority: int = priority


class GetDefaultPreferenceResult:
    """getDefaultPreference result

    Attributes:
        recipe_file_path (str): recipe file path 
        default_preference_version (int): DEFAULT_PRERENCE for the recipe file.
    """
    def __init__(
        self: "GetDefaultPreferenceResult",
        recipe_file_path: str,
        default_preference_version: int,
    ) -> None:
        self.recipe_file_path: str = recipe_file_path
        self.default_preference_version: int = default_preference_version


class GetSkippedRecipesResult:
    """getSkippedRecipes result

    Attributes:
        recipe_file_path (str): recipe file path 
        pn (str): packages name 
        skipreason (str): skipreason for the recipe file.
        provides (List[str]): package names provided by the recipe file 
        rprovides (List[str]): package alias names provided by the recipe file
    """
    def __init__(
        self: "GetSkippedRecipesResult", recipe_file_path: str, data: Union[Mapping[str, Any], "SkippedPackage"]
    ) -> None:
        self.recipe_file_path: str = recipe_file_path

        if isinstance(data, dict):
            self.pn: str = data.get("pn", None)
            self.skipreason: str = data.get("skipreason", None)
            self.provides: List[str] = data.get("provides", None)
            self.rprovides: List[str] = data.get("rprovides", None)
            return

        self.pn: str = data.pn if getattr(data, "pn") else None
        self.skipreason: str =  data.skipreason if getattr(data, "skipreason") else None
        self.provides: List[str] =  data.provides if getattr(data, "provides") else None
        self.rprovides: List[str] =  data.rprovides if getattr(data, "rprovides") else None

class GetAllAppendsResult:
    """getAllAppends result

    Attributes:
        target_recipe_name (str): recipe file name 
        append_file_path (str): append recipe file path for the recipe file name
    """
    def __init__(
        self: "GetAllAppendsResult", target_recipe_name: str, append_file_path: str
    ) -> None:
        self.target_recipe_name = target_recipe_name
        self.append_file_path = append_file_path


class FindProvidersResult:
    """findProviders result

    Attributes:
        package_name (str): recipe file name 
        latest_pe (List[Any]): latest package epoch
        latest_pv (List[Any]): latest package version
        latest_pr (List[Any]): latest package revision
        latest_recipe_file_path (List[Any]): latest package recipe file path
        preffered_pe (List[Any]): preffered package epoch
        preffered_pv (List[Any]): preffered package version
        preffered_pr (List[Any]): preffered package revision
        preffered_recipe_file_path (List[Any]): preffered package recipe file path
        required_version (Optional[bool]): whether required version exists or not. If the yocto version is old, it does not support this and will be None.
    """
    def __init__(
        self: "FindProvidersResult",
        package_name: str,
        latest_version: List[Any],
        perffered_version: List[Any],
        required_version: Optional[bool],
    ) -> None:
        self.package_name: str = package_name
        self.latest_pe: List[Any] = latest_version[0][0]
        self.latest_pv: List[Any] = latest_version[0][1]
        self.latest_pr: List[Any] = latest_version[0][2]
        self.latest_recipe_file_path: List[Any] = latest_version[1]
        self.preffered_pe: List[Any] = perffered_version[0][0]
        self.preffered_pv: List[Any] = perffered_version[0][1]
        self.preffered_pr: List[Any] = perffered_version[0][2]
        self.preffered_recipe_file_path: List[Any] = perffered_version[1]
        self.required_version: Optional[bool] = required_version


class AllProvidersResult:
    """allProviders result

    Attributes:
        package_name (str): package name
        recipes (List[GetRecipeVersionsResult]): recipe file path and its pe, pv, pr
    """
    def __init__(
        self: "AllProvidersResult", package_name: str, recipe_file: List[Any]
    ) -> None:
        self.package_name: str = package_name
        self.recipes: List[GetRecipeVersionsResult] = [
            GetRecipeVersionsResult(*i) for i in recipe_file
        ]


GetRuntimeRecommendsResult = GetRuntimeDependsResult

class JsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, getAllKeysWithFlagsResult):
            return obj.__dict__
        if isinstance(obj, GetLayerPrioritiesResult):
            return obj.__dict__
        if isinstance(obj, GetRecipesResult):
            return obj.__dict__
        if isinstance(obj, GetRecipeDependsResult):
            return obj.__dict__
        if isinstance(obj, GetRecipeVersionsResult):
            return obj.__dict__
        if isinstance(obj, GetRecipeProvidesResult):
            return obj.__dict__
        if isinstance(obj, GetRecipePackagesResult):
            return obj.__dict__
        if isinstance(obj, GetRecipePackagesDynamicResult):
            return obj.__dict__
        if isinstance(obj, GetRProvidersResult):
            return obj.__dict__
        if isinstance(obj, GetRuntimeDependsResult):
            return obj.__dict__
        if isinstance(obj, GetRuntimeRecommendsResult):
            return obj.__dict__
        if isinstance(obj, GetRecipeInheritsResult):
            return obj.__dict__
        if isinstance(obj, GetBbFilePriorityResult):
            return obj.__dict__
        if isinstance(obj, GetDefaultPreferenceResult):
            return obj.__dict__
        if isinstance(obj, GetSkippedRecipesResult):
            return obj.__dict__
        if isinstance(obj, GetAllAppendsResult):
            return obj.__dict__
        if isinstance(obj, FindProvidersResult):
            return obj.__dict__
        if isinstance(obj, AllProvidersResult):
            return obj.__dict__
        return json.JSONEncoder.default(self, obj)



#!/usr/bin/env python3
""" 
This file provides cli interface of bbclient
"""

from .bbclient import *

import logging, json
from argparse import ArgumentParser, Namespace, _SubParsersAction 
from logging import Logger, StreamHandler, getLogger
from typing import List, Union

from collections import namedtuple

Config = namedtuple(
    "Config",
    ["project_path", "subcommand", "command_args"],
)


def main() -> None:
    config: Config = get_config()
    if not config.subcommand:
        print("bbclient command needs subcommand. See bbclient --help.")
        return 
    logger: Logger = setup_logger()
    client: BBClient = BBClient(config.project_path, logger=logger)
    client.start_server()
    ui_handler: int = client.get_uihandler_num()
    client.set_event_mask(ui_handler, logging.DEBUG, {}, ["*"])
    client.parse_files()
    client.wait_done_async()

    config.subcommand(client, config.command_args)

    client.stop_server()

def get_config() -> List[Union[str, int]]:
    parser: ArgumentParser = ArgumentParser(description='')
    parser.add_argument("-p", "--project_path", default="../", help="path to bitbake project, basically poky dir")

    # sub commands
    sub_parsers: _SubParsersAction = parser.add_subparsers(title='get_all_keys_with_flags', description='valid subcommands', help='additional help')
    get_all_keys_with_flags_subcommand: ArgumentParser = sub_parsers.add_parser("get_all_keys_with_flags", help="Get value, history and specified flags of all global variables.")
    get_all_keys_with_flags_subcommand.add_argument("-f", "--flags", default=[], help="Target flags. The default is [].", action='append')
    get_all_keys_with_flags_subcommand.set_defaults(subcommand=get_all_keys_with_flags_command)

    get_variable_subcommand: ArgumentParser = sub_parsers.add_parser("get_variable", help="Get variable value.")
    get_variable_subcommand.add_argument("name", help="variable name")
    get_variable_subcommand.add_argument("-e", "--expand",  default=True, help="Whether to expand references to other variables. Defaults to True.")
    get_variable_subcommand.set_defaults(subcommand=get_variable)

    match_file_subcommand: ArgumentParser = sub_parsers.add_parser("match_file", help="Search file by regex.")
    match_file_subcommand.add_argument("file_path_regex", help="search regex pattern")
    match_file_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    match_file_subcommand.set_defaults(subcommand=match_file)

    get_layer_priorities_subcommand: ArgumentParser = sub_parsers.add_parser("get_layer_priorities", help="Get name, path, priority of all layers.")
    get_layer_priorities_subcommand.set_defaults(subcommand=get_layer_priorities)

    get_recipes_subcommand: ArgumentParser = sub_parsers.add_parser("get_recipes", help="Get all package name from cache.")
    get_recipes_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_recipes_subcommand.set_defaults(subcommand=get_recipes)

    get_recipe_depends_subcommand: ArgumentParser = sub_parsers.add_parser("get_recipe_depends", help="Get recipe depends.")
    get_recipe_depends_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_recipe_depends_subcommand.set_defaults(subcommand=get_recipe_depends)

    get_recipe_versions_subcommand: ArgumentParser = sub_parsers.add_parser("get_recipe_versions", help="Get all recipe versions.")
    get_recipe_versions_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_recipe_versions_subcommand.set_defaults(subcommand=get_recipe_versions)

    get_recipe_provides_subcommand: ArgumentParser = sub_parsers.add_parser("get_recipe_provides", help="Get all recipe files and its packages.")
    get_recipe_provides_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_recipe_provides_subcommand.set_defaults(subcommand=get_recipe_provides)

    get_recipe_packages_subcommand: ArgumentParser = sub_parsers.add_parser("get_recipe_packages", help="Get all recipe files and its recipe files.")
    get_recipe_packages_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_recipe_packages_subcommand.set_defaults(subcommand=get_recipe_packages)

    get_recipe_packages_dynamic_subcommand: ArgumentParser = sub_parsers.add_parser("get_recipe_packages_dynamic", help="Get all recipe files that provides PACKAGE_DYNAMIC and its PACKAGE_DYNAMIC.")
    get_recipe_packages_dynamic_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_recipe_packages_dynamic_subcommand.set_defaults(subcommand=get_recipe_packages_dynamic)

    get_r_providers_subcommand: ArgumentParser = sub_parsers.add_parser("get_r_providers", help="Get alias of PN and its recipe files.")
    get_r_providers_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_r_providers_subcommand.set_defaults(subcommand=get_r_providers)

    get_runtime_depends_subcommand: ArgumentParser = sub_parsers.add_parser("get_runtime_depends", help="Get all runtime dependency by all recipe files.")
    get_runtime_depends_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_runtime_depends_subcommand.set_defaults(subcommand=get_runtime_depends)

    get_runtime_recommends_subcommand: ArgumentParser = sub_parsers.add_parser("get_runtime_recommends", help="Get all runtime recoomends(=weak depends) by all recipe files.")
    get_runtime_recommends_subcommand.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_runtime_recommends_subcommand.set_defaults(subcommand=get_runtime_recommends)

    get_recipe_inherits_subcommands: ArgumentParser = sub_parsers.add_parser("get_recipe_inherits", help="Get recipes and its inherit recipes.")
    get_recipe_inherits_subcommands.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_recipe_inherits_subcommands.set_defaults(subcommand=get_recipe_inherits)
    
    get_bb_file_priority_subcommands: ArgumentParser = sub_parsers.add_parser("get_bb_file_priority", help="Get recipe files and its priority.")
    get_bb_file_priority_subcommands.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_bb_file_priority_subcommands.set_defaults(subcommand=get_bb_file_priority)

    get_default_preference_subcommands: ArgumentParser = sub_parsers.add_parser("get_default_preference", help="Get recipes and default preference.")
    get_default_preference_subcommands.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_default_preference_subcommands.set_defaults(subcommand=get_default_preference)

    get_skipped_recipes_subcommands: ArgumentParser = sub_parsers.add_parser("get_skipped_recipes", help="Get skipped recipes and its reasons, provides, alias.")
    get_skipped_recipes_subcommands.set_defaults(subcommand=get_skipped_recipes)

    get_file_appends_subcommands: ArgumentParser = sub_parsers.add_parser("get_file_appends", help="Get append files.")
    get_file_appends_subcommands.add_argument("file_path", help="recipe file path")
    get_file_appends_subcommands.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_file_appends_subcommands.set_defaults(subcommand=get_file_appends)

    get_all_appends_subcommands: ArgumentParser = sub_parsers.add_parser("get_all_appends", help="Get all append recipes.")
    get_all_appends_subcommands.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    get_all_appends_subcommands.set_defaults(subcommand=get_all_appends)

    find_providers_subcommands: ArgumentParser = sub_parsers.add_parser("find_providers", help="Get latest packages versions, prefered package versions, and whether there is an REQUIRED_VERSION.")
    find_providers_subcommands.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    find_providers_subcommands.set_defaults(subcommand=find_providers)

    find_best_provider_subcommands: ArgumentParser = sub_parsers.add_parser("find_best_provider", help="Get best provider infos.")
    find_best_provider_subcommands.add_argument("package_name", help="a package name you want to know the detail of best provider")
    find_best_provider_subcommands.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    find_best_provider_subcommands.set_defaults(subcommand=find_best_provider)

    all_providers_subcommands: ArgumentParser = sub_parsers.add_parser("all_providers", help="Get all providers versions and recipe file path.")
    all_providers_subcommands.add_argument("-m", "--mutli_conf_name",  default="", help="target multi-config. Defaults to ''")
    all_providers_subcommands.set_defaults(subcommand=all_providers)


    args: Namespace = parser.parse_args()

    # TODO: support remote server
    return Config(args.project_path, getattr(args, "subcommand", None), args)


def setup_logger() -> Logger:
    ch = StreamHandler()
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter('[%(name)s][%(asctime)s][%(levelname)s]: %(message)s')
    ch.setFormatter(formatter)
    logger: Logger = getLogger("bbclient")
    logger.setLevel('DEBUG')
    logger.addHandler(ch)
    return logger

def get_all_keys_with_flags_command(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[getAllKeysWithFlagsResult] = client.get_all_keys_with_flags(args.flags)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_variable(
    client: "BBClient", args: Namespace
) -> None:
    ret: str = client.get_variable(args.name, args.expand)
    print(ret)

def match_file(
    client: "BBClient", args: Namespace
) -> None:
    print("NOTE: This command can't run normaly because of bitbake bug. See https://angrymane.github.io/bbclient/bbclient.html#bbclient.bbclient.BBClient.match_file.")
    #ret: str = client.match_file(args.file_path_regex, args.mutli_conf_name)
    #print(ret)

def get_layer_priorities(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetLayerPrioritiesResult] = client.get_layer_priorities()
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_recipes(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRecipesResult] = client.get_recipes(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_recipe_depends(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRecipeDependsResult] = client.get_recipe_depends(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_recipe_versions(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRecipeVersionsResult] = client.get_recipe_versions(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_recipe_provides(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRecipeProvidesResult] = client.get_recipe_provides(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_recipe_packages(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRecipePackagesResult] = client.get_recipe_packages(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_recipe_packages_dynamic(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRecipePackagesDynamicResult] = client.get_recipe_packages_dynamic(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_r_providers(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRProvidersResult] = client.get_r_providers(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_runtime_depends(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRuntimeDependsResult] = client.get_runtime_depends(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_runtime_recommends(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRuntimeRecommendsResult] = client.get_runtime_recommends(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_recipe_inherits(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetRecipeInheritsResult] = client.get_recipe_inherits(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_bb_file_priority(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetBbFilePriorityResult] = client.get_bb_file_priority(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_default_preference(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetDefaultPreferenceResult] = client.get_default_preference(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_skipped_recipes(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetSkippedRecipesResult] = client.get_skipped_recipes()
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_file_appends(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[str] = client.get_file_appends(args.file_path, args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_all_appends(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[GetAllAppendsResult] = client.get_all_appends(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def find_providers(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[FindProvidersResult] = client.find_providers(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def find_best_provider(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[str] = client.find_best_provider(args.package_name, args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def all_providers(
    client: "BBClient", args: Namespace
) -> None:
    ret: List[AllProvidersResult] = client.all_providers(args.mutli_conf_name)
    json_str = json.dumps(ret, cls=JsonEncoder)
    print(json_str)

def get_runtime_providers(
    client: "BBClient", args: Namespace
) -> None:
    # TODO
    pass

def data_store_connector_cmd(
    client: "BBClient", args: Namespace
) -> None:
    # TODO
    pass

def data_store_connector_varhist_cmd(
    client: "BBClient", args: Namespace
) -> None:
    # TODO
    pass

"""
def data_store_connector_inc_hist_cmd(
    self: "BBClient", datastore_index: int, command: str, *args, **kwargs
) -> Any:
def data_store_connector_release(self: "BBClient", datastore_index: int) -> None:
def parse_recipe_file(
    self: "BBClient",
    file_path: str,
    append: bool = True,
    append_list: Optional[str] = None,
    datastore_index: Optional[int] = None,
) -> Optional[int]:
def build_file(
    self: "BBClient", file_path: str, task_name: str, internal: bool = False
) -> None:
def build_targets(
    self: "BBClient",
    targets: List[str],
    task_name: str,
) -> None:
def generate_dep_tree_event(
    self: "BBClient", targets: List[str], task_name: str
) -> None:
def generate_dot_graph(
    self: "BBClient", targets: List[str], task_name: str
) -> None:
def generate_targets_tree(
    self: "BBClient", bb_klass_file_path: str, package_names: List[str]
) -> None:
def find_config_files(self: "BBClient", variable_name: str) -> None:
def find_files_matching_in_dir(
    self: "BBClient", target_file_name_substring: str, directory: str
) -> None:
def find_config_file_path(self: "BBClient", config_file_name: str) -> None:
def show_versions(self: "BBClient") -> None:
def show_environment_target(self: "BBClient", package_name: str = "") -> None:
def show_environment(self: "BBClient", bb_file_path: str) -> None:
def parse_files(self: "BBClient") -> None:

def compare_revisions(self: "BBClient") -> None:
"""



"""
from test_get_all_keys_with_flags import *
from test_get_variable import *
from test_set_variable import *
from test_get_set_variable import *
from test_set_config import *
from test_enable_data_tracking import *
from test_disable_data_tracking import *
from test_set_pre_post_conf_files import *
from test_match_file import *
from test_get_uihandler_num import *
from test_set_event_mask import *
from test_set_features import *
from test_update_config import *
from test_parse_configuration import *
from test_get_layer_priorities import *
from test_get_recipes import *
from test_get_recipe_depends import *
from test_get_recipe_versions import *
from test_get_recipe_provides import *
from test_get_recipe_packages import *
from test_get_recipe_packages_dynamic import *
from test_get_r_providers import *
from test_get_runtime_depends import *
from test_get_runtime_recommends import *
from test_get_recipe_inherits import *
from test_get_bb_file_priority import *
from test_get_default_preference import *
from test_get_skipped_recipes import *
from test_get_overlayed_recipes import *
from test_get_file_appends import *
from test_get_all_appends import *
from test_find_providers import *
from test_find_best_provider import *
from test_all_providers import *
from test_get_runtime_providers import *
from test_data_store_connector_cmd import *
from test_data_store_connector_varhist_cmd import *
from test_data_store_connector_var_hist_cmd_emit import *
from test_data_store_connector_inc_hist_cmd import *
from test_data_store_connector_release import *
from test_parse_recipe_file import *
from test_build_file import *
from test_build_targets import *
from test_generate_dep_tree_event import *
from test_generate_dot_graph import *
from test_generate_targets_tree import *
from test_find_config_files import *
from test_find_files_matching_in_dir import *
from test_test_cooker_command_event import *
from test_find_config_file_path import *
from test_show_versions import *
from test_show_environment_target import *
from test_show_environment import *
from test_parse_files import *
from test_compare_revisions import *
from test_trigger_event import *
from test_reset_cooker import *
from test_client_complete import *
from test_find_sigInfo import *
"""


from setuptools import setup, find_packages
from os import path


def read(fname):
    return open(path.join(path.dirname(__file__), fname)).read()


setup(
    name="bbclient",
    version="0.7.1",
    description="bbclient provides utility commands for bitbake.",
    long_description_content_type="text/markdown",
    long_description=read("README.md"),
    keywords=["yocto", "bitbake"],
    url="https://github.com/AngryMane/bbclient",
    author="AngryMane",
    author_email="regulationdango@gmail.com",
    license="MIT",
    python_requires=">=3.7",
    install_requires=["bbclient"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Plugins",
        "Topic :: Software Development :: Build Tools",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
    ],
    entry_points={
        "console_scripts": [
            "bbclient=bbclient.console:main",
        ],
    },
    packages=find_packages(),
)



from .common import *
from bbclient import *

params = [
    (["gcc"], "compile"),
    (["busybox", "python3:do_fetch"], "compile"),
]

@pytest.mark.parametrize("targets, task", params)
def test_generate_dot_graph_main(main_client: BBClient, targets: List[str], task: str) -> None:
    __test_impl(main_client, targets, task)

@pytest.mark.parametrize("targets, task", params)
def test_generate_dot_graph_kirkstone(kirkstone_client: BBClient, targets: List[str], task: str) -> None:
    __test_impl(kirkstone_client, targets, task)

@pytest.mark.parametrize("targets, task", params)
def test_generate_dot_graph_dunfell(dunfell_client: BBClient, targets: List[str], task: str) -> None:
    __test_impl(dunfell_client, targets, task)

def __test_impl(client: BBClient, targets: List[str], task: str) -> None:
    client.generate_dot_graph(targets, task)
    ret: Optional[BBEventBase] = client.wait_event([TreeDataPreparationStartedEvent])
    assert isinstance(ret, TreeDataPreparationStartedEvent)
    ret: Optional[BBEventBase] = client.wait_event([TreeDataPreparationProgressEvent])
    assert isinstance(ret, TreeDataPreparationProgressEvent)
    ret: Optional[BBEventBase] = client.wait_event([TreeDataPreparationCompletedEvent])
    assert isinstance(ret, TreeDataPreparationCompletedEvent)
    ret: Optional[BBEventBase] = client.wait_done_async()
    assert isinstance(ret, CommandCompletedEvent)



from .common import *
from bbclient import *

def test_get_uihandler_num_main(main_client: BBClient) -> None:
    __test_impl(main_client)

def test_get_uihandler_num_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_get_uihandler_num_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    ret: int = client.get_uihandler_num()
    assert ret == 1


from .common import *
from bbclient import *

params = [
    (".conf", "recipes-bsp", [FilesMatchingFoundEvent], CommandCompletedEvent),
    (".conf", "recipes-dummy_dir", [], CommandCompletedEvent),
    (".conffff", "recipes-bsp", [], CommandCompletedEvent),
]

@pytest.mark.parametrize("file_regex, directory, expect_event_types, expect_command_result", params)
def test_find_files_matching_in_dir_main(main_client: BBClient, file_regex: str, directory: str, expect_event_types: List[BBEventBase], expect_command_result: BBEventBase) -> None:
    __test_impl(main_client, file_regex, directory, expect_event_types, expect_command_result)
    
@pytest.mark.parametrize("file_regex, directory, expect_event_types, expect_command_result", params)
def test_find_config_files_kirkstone(kirkstone_client: BBClient, file_regex: str, directory: str, expect_event_types: List[BBEventBase], expect_command_result: BBEventBase) -> None:
    __test_impl(kirkstone_client, file_regex, directory, expect_event_types, expect_command_result)

@pytest.mark.parametrize("file_regex, directory, expect_event_types, expect_command_result", params)
def test_find_config_files_dunfell(dunfell_client: BBClient, file_regex: str, directory: str, expect_event_types: List[BBEventBase], expect_command_result: BBEventBase) -> None:
    __test_impl(dunfell_client, file_regex, directory, expect_event_types, expect_command_result)

def __test_impl(client: BBClient, file_regex: str, directory: str, expect_event_types: List[BBEventBase], expect_command_result: BBEventBase) -> None:
    client.find_files_matching_in_dir(file_regex, directory)
    for expect_event_type in expect_event_types:
        ret: Optional[BBEventBase] = client.wait_event([expect_event_type])
        assert isinstance(ret, FilesMatchingFoundEvent)
    ret: Optional[BBEventBase] = client.wait_done_async()
    assert isinstance(ret, expect_command_result)


from .common import *
from bbclient import *

def test_get_all_appends_main(main_client: BBClient) -> None:
    __test_impl(main_client)

def test_get_all_appends_kirkstone(kirkstone_client: BBClient) -> None:
    __test_impl(kirkstone_client)

def test_get_all_appends_dunfell(dunfell_client: BBClient) -> None:
    __test_impl(dunfell_client)

def __test_impl(client: BBClient) -> None:
    ret: List[GetAllAppendsResult] = client.get_all_appends()
    assert ret is not None
    assert len(ret) != 0